@@ -31,8 +31,8 @@
 /* Define to 1 if you have the <strings.h> header file. */
 #cmakedefine HAVE_STRINGS_H 1
 
-/* Define to 1 if you have the <SDL/SDL_config.h> header file. */
-#cmakedefine HAVE_SDL_SDL_CONFIG_H 1
+/* Define to 1 if you have the <SDL_config.h> header file. */
+#cmakedefine HAVE_SDL_CONFIG_H 1
 
 /* Define to 1 if you have the <sys/times.h> header file. */
 #cmakedefine HAVE_SYS_TIMES_H 1
@@ -76,6 +76,14 @@
 /* Define to 1 if you have the 'ftello' function. */
 #cmakedefine HAVE_FTELLO 1
 
+/* Define to 1 if you have the 'flock' function. */
+#cmakedefine HAVE_FLOCK 1
+
+/* Define to 1 if you have the 'strlcpy' function. */
+#cmakedefine HAVE_LIBC_STRLCPY 1
+
+/* Define to 1 if you have the 'd_type' member in the 'dirent' struct */
+#cmakedefine HAVE_DIRENT_D_TYPE 1
 
 /* Relative path from bindir to datadir */
 #define BIN2DATADIR "@BIN2DATADIR@"
