@@ -100,8 +100,9 @@ bool Change_DoNeedReset(CNF_PARAMS *current, CNF_PARAMS *changed)
 	    || strcmp(changed->HardDisk.szIdeSlaveHardDiskImage, current->HardDisk.szIdeSlaveHardDiskImage))
 		return true;
 
-	/* Did change GEMDOS drive? */
-	if (changed->HardDisk.bUseHardDiskDirectories != current->HardDisk.bUseHardDiskDirectories
+	/* Did change GEMDOS drive Atari/host location or enabling? */
+	if (changed->HardDisk.nGemdosDrive != current->HardDisk.nGemdosDrive
+	    || changed->HardDisk.bUseHardDiskDirectories != current->HardDisk.bUseHardDiskDirectories
 	    || (strcmp(changed->HardDisk.szHardDiskDirectories[0], current->HardDisk.szHardDiskDirectories[0])
 	        && changed->HardDisk.bUseHardDiskDirectories))
 		return true;
@@ -145,6 +146,10 @@ bool Change_DoNeedReset(CNF_PARAMS *current, CNF_PARAMS *changed)
 	/* Did change FPU? */
 	if (changed->System.n_FPUType != current->System.n_FPUType)
 		return true;
+
+	/* Did change size of TT-RAM? */
+	if (current->Memory.nTTRamSize != changed->Memory.nTTRamSize)
+		return true;
 #endif
 
 	/* Did change size of memory? */
@@ -192,7 +197,12 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 	     || changed->Screen.nMaxWidth != current->Screen.nMaxWidth
 	     || changed->Screen.nMaxHeight != current->Screen.nMaxHeight
 	     || changed->Screen.bAllowOverscan != current->Screen.bAllowOverscan
-	     || changed->Screen.bShowStatusbar != current->Screen.bShowStatusbar))
+	     || changed->Screen.bShowStatusbar != current->Screen.bShowStatusbar
+#if WITH_SDL2
+	     || changed->Screen.nRenderScaleQuality != current->Screen.nRenderScaleQuality
+	     || changed->Screen.bUseVsync != current->Screen.bUseVsync
+#endif
+	    ))
 	{
 		Dprintf("- screenmode>\n");
 		bScreenModeChange = true;
@@ -252,8 +262,9 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 	if ( changed->DiskImage.DriveB_NumberOfHeads != current->DiskImage.DriveB_NumberOfHeads )
 		FDC_Drive_Set_NumberOfHeads ( 1 , changed->DiskImage.DriveB_NumberOfHeads );
 
-	/* Did change GEMDOS drive? */
-	if (changed->HardDisk.bUseHardDiskDirectories != current->HardDisk.bUseHardDiskDirectories
+	/* Did change GEMDOS drive Atari/host location or enabling? */
+	if (changed->HardDisk.nGemdosDrive != current->HardDisk.nGemdosDrive
+	    || changed->HardDisk.bUseHardDiskDirectories != current->HardDisk.bUseHardDiskDirectories
 	    || (strcmp(changed->HardDisk.szHardDiskDirectories[0], current->HardDisk.szHardDiskDirectories[0])
 	        && changed->HardDisk.bUseHardDiskDirectories))
 	{
@@ -315,7 +326,7 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 	    changed->System.nDSPType != DSP_TYPE_EMU)
 	{
 		Dprintf("- DSP>\n");
-		DSP_UnInit();
+		DSP_Disable();
 	}
 #endif
 
@@ -346,7 +357,7 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 	    changed->System.nDSPType == DSP_TYPE_EMU)
 	{
 		Dprintf("- DSP<\n");
-		DSP_Init();
+		DSP_Enable();
 	}
 #endif
 
@@ -427,7 +438,7 @@ void Change_CopyChangedParamsToConfiguration(CNF_PARAMS *current, CNF_PARAMS *ch
 	if (bScreenModeChange)
 	{
 		Dprintf("- screenmode<\n");
-		Screen_ModeChanged();
+		Screen_ModeChanged(true);
 	}
 
 	/* Do we need to perform reset? */
