@@ -14,18 +14,16 @@
 
 extern int pendingInterrupts;
 
+extern void customreset(void);
+extern int intlev (void);
 extern int Init680x0(void);
 extern void Exit680x0(void);
-#ifndef ENABLE_WINUAE_CPU
-extern void customreset(void);
-#endif
-extern int intlev (void);
 
-extern unsigned long OpCode_GemDos(uae_u32 opcode);
-extern unsigned long OpCode_SysInit(uae_u32 opcode);
-extern unsigned long OpCode_VDI(uae_u32 opcode);
-extern unsigned long OpCode_NatFeat_ID(uae_u32 opcode);
-extern unsigned long OpCode_NatFeat_Call(uae_u32 opcode);
+extern uae_u32 OpCode_GemDos(uae_u32 opcode);
+extern uae_u32 OpCode_SysInit(uae_u32 opcode);
+extern uae_u32 OpCode_VDI(uae_u32 opcode);
+extern uae_u32 OpCode_NatFeat_ID(uae_u32 opcode);
+extern uae_u32 OpCode_NatFeat_Call(uae_u32 opcode);
 
 #define write_log printf
 
