@@ -127,15 +127,9 @@ static int Cycles_GetInternalCycleOnReadAccess(void)
 	}
 	else							/* BUS_MODE_CPU */
 	{
-		/* TODO: Find proper cycles count depending on the type of the current instruction */
+		/* TODO: Find proper cycles count depending on the opcode/family of the current instruction */
 		/* (e.g. movem is not correctly handled) */
-		/* We don't read the opcode if PC is located in the IO region (rare cases */
-		/* used in some games/demos protections) as this can create recursive calls */
-		/* (protection of the "Union Demo" runs at $ff8240) */
-		if ( ( ( BusErrorPC & 0xffffff ) < 0xff0000 ) || ( ( BusErrorPC & 0xffffff ) > 0xffffff ) )
-			Opcode = get_word(BusErrorPC);					/* BusErrorPC points to the current opcode */
-		else
-			Opcode = -1;
+		Opcode = M68000_CurrentOpcode;
 		//fprintf ( stderr , "opcode=%x\n" , Opcode );
 
 		/* Assume we use 'move src,dst' : access cycle depends on dst mode */
@@ -144,6 +138,7 @@ static int Cycles_GetInternalCycleOnReadAccess(void)
 		else if ( OpcodeFamily == i_MVPRM )					/* eg movep.l d0,$ffc3(a1) in E605 (STE) */
 			AddCycles = 12 + MovepByteNbr * 4;				/* [NP] FIXME, it works with E605 but gives 20-32 cycles instead of 16-28 */
 											/* something must be wrong in video.c */
+			/* FIXME : this should be : AddCycles = 4 + MovepByteNbr * 4, but this breaks e605 in video.c */
 		else
 			AddCycles = CurrentInstrCycles + nWaitStateCycles;		/* assume dest is reg : read is effective at the end of the instr */
 	}
