@@ -37,7 +37,7 @@ extern bool DebugUI_ParseFile(const char *path, bool reinit);
 extern int DebugDsp_Init(const dbgcommand_t **table);
 extern void DebugDsp_InitSession(void);
 #else /* !ENABLE_DSP_EMU */
-static inline int DebugDsp_Init(const dbgcommand_t **t) { t = NULL; return 0; }
+static inline int DebugDsp_Init(const dbgcommand_t **t) { *t = NULL; return 0; }
 #define DebugDsp_InitSession()
 #endif /* !ENABLE_DSP_EMU */
 
