@@ -67,6 +67,7 @@ typedef enum
 #define __attribute__(foo)
 #endif
 
+extern void Log_Default(void);
 extern int Log_Init(void);
 extern int Log_SetAlertLevel(int level);
 extern void Log_UnInit(void);
@@ -163,6 +164,14 @@ extern char *Log_MatchTrace(const char *text, int state);
 
 #define TRACE_NATFEATS		 (1ll<<47)
 
+#define TRACE_KEYMAP		 (1ll<<48)
+
+#define TRACE_MIDI		 (1ll<<49)
+
+#define TRACE_IDE		 (1ll<<50)
+
+#define TRACE_OS_BASE		 (1ll<<51)
+
 #define	TRACE_NONE		 (0)
 #define	TRACE_ALL		 (~0)
 
@@ -179,7 +188,7 @@ extern char *Log_MatchTrace(const char *text, int state);
 
 #define	TRACE_IKBD_ALL		( TRACE_IKBD_CMDS | TRACE_IKBD_ACIA | TRACE_IKBD_EXEC )
 
-#define	TRACE_OS_ALL		( TRACE_OS_BIOS | TRACE_OS_XBIOS | TRACE_OS_GEMDOS | TRACE_OS_AES | TRACE_OS_VDI )
+#define	TRACE_OS_ALL		( TRACE_OS_BASE | TRACE_OS_BIOS | TRACE_OS_XBIOS | TRACE_OS_GEMDOS | TRACE_OS_AES | TRACE_OS_VDI )
 
 #define	TRACE_IOMEM_ALL		( TRACE_IOMEM_RD | TRACE_IOMEM_WR )
 
@@ -193,9 +202,9 @@ extern Uint64 LogTraceFlags;
 
 #ifndef _VCWIN_
 #define	LOG_TRACE(level, args...) \
-	if (unlikely(LogTraceFlags & level)) { fprintf(TraceFile, args); fflush(TraceFile); }
+	if (unlikely(LogTraceFlags & (level))) { fprintf(TraceFile, args); fflush(TraceFile); }
 #endif
-#define LOG_TRACE_LEVEL( level )	(unlikely(LogTraceFlags & level))
+#define LOG_TRACE_LEVEL( level )	(unlikely(LogTraceFlags & (level)))
 
 #else		/* ENABLE_TRACING */
 
