@@ -17,7 +17,7 @@ extern bool VIDEL_renderScreen(void);
 
 extern void VIDEL_reset(void);
 
-extern void VIDEL_ZoomModeChanged(void);
+extern void VIDEL_ZoomModeChanged(bool bForceChange);
 extern void VIDEL_ConvertScreenNoZoom(int vw, int vh, int bpp, int nextline);
 extern void VIDEL_ConvertScreenZoom(int vw, int vh, int bpp, int nextline);
 
@@ -78,6 +78,6 @@ extern void VIDEL_InterruptHandler_HalfLine(void);
 /* Called from memorySnapShot.c */
 extern void VIDEL_MemorySnapShot_Capture(bool bSave);
 
-extern void Videl_Info(Uint32 dummy);
+extern void Videl_Info(FILE *fp, Uint32 dummy);
 
 #endif /* _VIDEL_H */
