@@ -36,20 +36,20 @@ static SGOBJ newdiskdlg[] =
 	{ SGBOX, 0, 0, 0,0, 29,14, NULL },
 	{ SGTEXT, 0, 0, 6,1, 16,1, "New floppy image" },
 	{ SGTEXT, 0, 0, 2,3, 7,1, "Tracks:" },
-	{ SGBUTTON, 0, 0, 12,3, 1,1, "\x04" },   /* Left-arrow button  */
+	{ SGBUTTON, 0, 0, 12,3, 1,1, "\x04", SG_SHORTCUT_LEFT },
 	{ SGTEXT, 0, 0, 14,3, 2,1, szTracks },
-	{ SGBUTTON, 0, 0, 17,3, 1,1, "\x03" },   /* Right-arrow button */
+	{ SGBUTTON, 0, 0, 17,3, 1,1, "\x03", SG_SHORTCUT_RIGHT },
 	{ SGTEXT, 0, 0, 2,5, 8,1, "Sectors:" },
-	{ SGRADIOBUT, 0, SG_SELECTED, 12,5, 4,1, " 9" },
-	{ SGRADIOBUT, 0, 0, 17,5, 4,1, "10" },
+	{ SGRADIOBUT, 0, SG_SELECTED, 12,5, 4,1, " _9" },
+	{ SGRADIOBUT, 0, 0, 17,5, 4,1, "1_0" },
 	{ SGRADIOBUT, 0, 0, 22,5, 4,1, "11" },
-	{ SGRADIOBUT, 0, 0, 12,6, 9,1, "18 (HD)" },
-	{ SGRADIOBUT, 0, 0, 12,7, 9,1, "36 (ED)" },
+	{ SGRADIOBUT, 0, 0, 12,6, 9,1, "1_8 (HD)" },
+	{ SGRADIOBUT, 0, 0, 12,7, 9,1, "3_6 (ED)" },
 	{ SGTEXT, 0, 0, 2,9, 6,1, "Sides:" },
-	{ SGRADIOBUT, 0, 0, 12,9, 4,1, "1" },
-	{ SGRADIOBUT, 0, SG_SELECTED, 17,9, 4,1, "2" },
-	{ SGBUTTON, 0, 0, 4,12, 8,1, "Create" },
-	{ SGBUTTON, 0, 0, 18,12, 6,1, "Back" },
+	{ SGRADIOBUT, 0, 0, 12,9, 3,1, "_1" },
+	{ SGRADIOBUT, 0, SG_SELECTED, 17,9, 3,1, "_2" },
+	{ SGBUTTON, SG_DEFAULT, 0, 4,12, 8,1, "_Create" },
+	{ SGBUTTON, SG_CANCEL, 0, 18,12, 6,1, "_Back" },
 	{ -1, 0, 0, 0,0, 0,0, NULL }
 };
 
@@ -116,7 +116,7 @@ char *DlgNewDisk_Main(void)
 	/* Draw and process the dialog */
 	do
 	{
-		but = SDLGui_DoDialog(newdiskdlg, NULL);
+		but = SDLGui_DoDialog(newdiskdlg, NULL, false);
 		switch(but)
 		{
 		 case DLGNEWDISK_DECTRACK:
@@ -130,7 +130,7 @@ char *DlgNewDisk_Main(void)
 			sprintf(szTracks, "%i", nTracks);
 			break;
 		 case DLGNEWDISK_SAVE:
-			tmpname = SDLGui_FileSelect(szNewDiskName, NULL, true);
+			tmpname = SDLGui_FileSelect("New floppy image:", szNewDiskName, NULL, true);
 			if (tmpname)
 			{
 				if (DlgNewDisk_CreateDisk(tmpname))
