@@ -57,37 +57,37 @@ static SGOBJ systemdlg[] =
 
 	{ SGBOX, 0, 0, 2,3, 14,8, NULL },
 	{ SGTEXT, 0, 0, 3,3, 13,1, "Machine type" },
-	{ SGRADIOBUT, 0, 0, 3,5, 8,1, "ST" },
-	{ SGRADIOBUT, 0, 0, 3,6, 8,1, "STE" },
-	{ SGRADIOBUT, 0, 0, 3,7, 8,1, "TT" },
-	{ SGRADIOBUT, 0, 0, 3,8, 8,1, "Falcon" },
+	{ SGRADIOBUT, 0, 0, 3,5, 4,1, "_ST" },
+	{ SGRADIOBUT, 0, 0, 3,6, 5,1, "ST_E" },
+	{ SGRADIOBUT, 0, 0, 3,7, 4,1, "_TT" },
+	{ SGRADIOBUT, 0, 0, 3,8, 8,1, "_Falcon" },
 
 	{ SGBOX, 0, 0, 17,3, 13,8, NULL },
 	{ SGTEXT, 0, 0, 19,3, 8,1, "CPU type" },
-	{ SGRADIOBUT, 0, 0, 19,5, 7,1, "68000" },
-	{ SGRADIOBUT, 0, 0, 19,6, 7,1, "68010" },
-	{ SGRADIOBUT, 0, 0, 19,7, 7,1, "68020" },
+	{ SGRADIOBUT, 0, 0, 19, 5, 7,1, "680_00" },
+	{ SGRADIOBUT, 0, 0, 19, 6, 7,1, "680_10" },
+	{ SGRADIOBUT, 0, 0, 19, 7, 7,1, "680_20" },
 #ifdef ENABLE_WINUAE_CPU
-	{ SGRADIOBUT, 0, 0, 19,8, 7,1, "68030" },
-	{ SGRADIOBUT, 0, 0, 19,9, 7,1, "68040" },
+	{ SGRADIOBUT, 0, 0, 19, 8, 7,1, "680_30" },
+	{ SGRADIOBUT, 0, 0, 19, 9, 7,1, "680_40" },
 	{ SGRADIOBUT, 0, 0, 19,10, 7,1, "68060" },
 #else
-	{ SGRADIOBUT, 0, 0, 19,8, 13,1, "68030+FPU" },
-	{ SGRADIOBUT, 0, 0, 19,9, 7,1, "68040" },
+	{ SGRADIOBUT, 0, 0, 19, 8,11,1, "680_30+FPU" },
+	{ SGRADIOBUT, 0, 0, 19, 9, 7,1, "680_40" },
 	{ SGTEXT, 0, 0, 19,10, 7,1, "" },
 #endif
 
 	{ SGBOX, 0, 0, 31,3, 12,8, NULL },
 	{ SGTEXT, 0, 0, 32,3, 15,1, "CPU clock" },
-	{ SGRADIOBUT, 0, 0, 32,5, 8,1, " 8 Mhz" },
-	{ SGRADIOBUT, 0, 0, 32,6, 8,1, "16 Mhz" },
-	{ SGRADIOBUT, 0, 0, 32,7, 8,1, "32 Mhz" },
+	{ SGRADIOBUT, 0, 0, 32,5, 8,1, " _8 Mhz" },
+	{ SGRADIOBUT, 0, 0, 32,6, 8,1, "1_6 Mhz" },
+	{ SGRADIOBUT, 0, 0, 32,7, 8,1, "32 _Mhz" },
 
 	{ SGBOX, 0, 0, 44,3, 14,8, NULL },
 	{ SGTEXT, 0, 0, 45,3, 11,1, "Falcon DSP" },
-	{ SGRADIOBUT, 0, 0, 45,5, 6,1, "None" },
-	{ SGRADIOBUT, 0, 0, 45,6, 7,1, "Dummy" },
-	{ SGRADIOBUT, 0, 0, 45,7, 6,1, "Full" },
+	{ SGRADIOBUT, 0, 0, 45,5, 6,1, "_None" },
+	{ SGRADIOBUT, 0, 0, 45,6, 7,1, "Dumm_y" },
+	{ SGRADIOBUT, 0, 0, 45,7, 6,1, "Ful_l" },
 
 #ifdef ENABLE_WINUAE_CPU
 	{ SGBOX, 0, 0, 2,12, 41,10, NULL },
@@ -95,15 +95,15 @@ static SGOBJ systemdlg[] =
 	{ SGBOX, 0, 0, 2,12, 56,9, NULL },
 #endif
 	{ SGTEXT, 0, 0, 3,12, 11,1, "CPU & system parameters" },
-	{ SGCHECKBOX, 0, 0, 3,14, 27,1, "Real time clock emulation" },
-	{ SGCHECKBOX, 0, 0, 3,15, 20,1, "Blitter emulation" },
-	{ SGCHECKBOX, 0, 0, 3,16, 15,1, "Patch Timer-D" },
-	{ SGCHECKBOX, 0, 0, 3,17, 39,1, "Boot faster by patching TOS & sysvars" },
-	{ SGCHECKBOX, 0, 0, 3,18, 25,1, "Prefetch mode, slower" },
+	{ SGCHECKBOX, 0, 0, 3,14, 27,1, "_Real time clock emulation" },
+	{ SGCHECKBOX, 0, 0, 3,15, 19,1, "_Blitter emulation" },
+	{ SGCHECKBOX, 0, 0, 3,16, 15,1, "Patch Timer-_D" },
+	{ SGCHECKBOX, 0, 0, 3,17, 39,1, "Boot faster by _patching TOS & sysvars" },
+	{ SGCHECKBOX, 0, 0, 3,18, 23,1, "Prefetc_h mode, slower" },
 #ifdef ENABLE_WINUAE_CPU
-	{ SGCHECKBOX, 0, 0, 3,19, 22,1, "Cycle exact, slower" },
-	{ SGCHECKBOX, 0, 0, 3,20, 15,1, "MMU emulation" },
-	{ SGCHECKBOX, 0, 0, 3,21, 21,1, "24 bits addressing" },
+	{ SGCHECKBOX, 0, 0, 3,19, 22,1, "Cycle e_xact, slower" },
+	{ SGCHECKBOX, 0, 0, 3,20, 15,1, "MM_U emulation" },
+	{ SGCHECKBOX, 0, 0, 3,21, 21,1, "24 bits addressin_g" },
 #else
 	{ SGTEXT, 0, 0, 3,19, 1,1, "" },
 	{ SGTEXT, 0, 0, 3,20, 1,1, "" },
@@ -113,12 +113,12 @@ static SGOBJ systemdlg[] =
 #ifdef ENABLE_WINUAE_CPU
 	{ SGBOX, 0, 0, 44,12, 14,10, NULL },
 	{ SGTEXT, 0, 0, 45,12, 11,1, "FPU" },
-	{ SGRADIOBUT, 0, 0, 45,14, 6,1, "None" },
+	{ SGRADIOBUT, 0, 0, 45,14, 6,1, "N_one" },
 	{ SGRADIOBUT, 0, 0, 45,15, 7,1, "68881" },
 	{ SGRADIOBUT, 0, 0, 45,16, 7,1, "68882" },
-	{ SGRADIOBUT, 0, 0, 45,17, 14,1, "internal" },
-	{ SGCHECKBOX, 0, 0, 45,19, 25,1, "compatible" },
-	{ SGTEXT, 0, 0, 47,20, 12,1, "but slower" },
+	{ SGRADIOBUT, 0, 0, 45,17, 14,1, "_internal" },
+	{ SGCHECKBOX, 0, 0, 45,19, 25,1, "_compatible" },
+	{ SGTEXT,     0, 0, 47,20, 12,1, "but slower" },
 #else
 	{ SGTEXT, 0, 0, 44,12, 14,10, "" },
 	{ SGTEXT, 0, 0, 45,12, 11,1, "" },
@@ -160,7 +160,7 @@ void Dialog_SystemDlg(void)
 	systemdlg[DLGSYS_ST + ConfigureParams.System.nMachineType].state |= SG_SELECTED;
 
 	/* CPU frequency: */
-	for (i = DLGSYS_8MHZ; i <= DLGSYS_16MHZ; i++)
+	for (i = DLGSYS_8MHZ; i <= DLGSYS_32MHZ; i++)
 	{
 		systemdlg[i].state &= ~SG_SELECTED;
 	}
@@ -254,7 +254,7 @@ void Dialog_SystemDlg(void)
 #endif
 
 	/* Show the dialog: */
-	SDLGui_DoDialog(systemdlg, NULL);
+	SDLGui_DoDialog(systemdlg, NULL, false);
 
 	/* Read values from dialog: */
 
