@@ -8,6 +8,8 @@
 */
 const char HDC_fileid[] = "Hatari hdc.c : " __DATE__ " " __TIME__;
 
+#include <errno.h>
+
 #include "main.h"
 #include "configuration.h"
 #include "debugui.h"
@@ -44,6 +46,7 @@ const char HDC_fileid[] = "Hatari hdc.c : " __DATE__ " " __TIME__;
 
 // #define DISALLOW_HDC_WRITE
 // #define HDC_VERBOSE           /* display operations */
+#define WITH_NCR5380 0
 
 #define HDC_ReadInt16(a, i) (((unsigned) a[i] << 8) | a[i + 1])
 #define HDC_ReadInt24(a, i) (((unsigned) a[i] << 16) | ((unsigned) a[i + 1] << 8) | a[i + 2])
@@ -71,14 +74,21 @@ typedef struct {
 	Uint8 opcode;
 	bool bDmaError;
 	short int returnCode;       /* return code from the HDC operation */
+	Uint8 *resp;                /* Response buffer */
+	int respbufsize;
+	int respcnt;
+	int respidx;
 	SCSI_DEV devs[8];
 } SCSI_CTRLR;
 
 /* HDC globals */
-SCSI_CTRLR AcsiBus;
-int nPartitions = 0;
+static SCSI_CTRLR AcsiBus;
+int nAcsiPartitions = 0;
 bool bAcsiEmuOn = false;
 
+#if WITH_NCR5380
+static SCSI_CTRLR ScsiBus;
+#endif
 
 /* Our dummy INQUIRY response data */
 static unsigned char inquiry_bytes[] =
@@ -141,6 +151,24 @@ static inline Uint8 HDC_GetControl(SCSI_CTRLR *ctr)
 		return ctr->command[9];			/* Class 1 */
 }
 
+/**
+ * Get pointer to response buffer, set up size indicator - and allocate
+ * a new buffer if it is not big enough
+ */
+static Uint8 *HDC_PrepRespBuf(SCSI_CTRLR *ctr, int size)
+{
+	ctr->respcnt = size;
+	ctr->respidx = 0;
+
+	if (size > ctr->respbufsize)
+	{
+		ctr->respbufsize = size;
+		ctr->resp = realloc(ctr->resp, size);
+	}
+
+	return ctr->resp;
+}
+
 /**
  * Get info string for SCSI/ACSI command packets.
  */
@@ -165,17 +193,19 @@ static void HDC_Cmd_Seek(SCSI_CTRLR *ctr)
 
 	dev->nLastBlockAddr = HDC_GetLBA(ctr);
 
-	LOG_TRACE(TRACE_SCSI_CMD, "HDC: SEEK (%s), LBA=%i.\n",
+	LOG_TRACE(TRACE_SCSI_CMD, "HDC: SEEK (%s), LBA=%i",
 	          HDC_CmdInfoStr(ctr), dev->nLastBlockAddr);
 
 	if (dev->nLastBlockAddr < dev->hdSize &&
 	    fseeko(dev->image_file, (off_t)dev->nLastBlockAddr * 512L, SEEK_SET) == 0)
 	{
+		LOG_TRACE(TRACE_SCSI_CMD, " -> OK\n");
 		ctr->returnCode = HD_STATUS_OK;
 		dev->nLastError = HD_REQSENS_OK;
 	}
 	else
 	{
+		LOG_TRACE(TRACE_SCSI_CMD, " -> ERROR\n");
 		ctr->returnCode = HD_STATUS_ERROR;
 		dev->nLastError = HD_REQSENS_INVADDR;
 	}
@@ -190,35 +220,26 @@ static void HDC_Cmd_Seek(SCSI_CTRLR *ctr)
 static void HDC_Cmd_Inquiry(SCSI_CTRLR *ctr)
 {
 	SCSI_DEV *dev = &ctr->devs[ctr->target];
-	Uint32 nDmaAddr;
+	Uint8 *buf;
 	int count;
 
-	nDmaAddr = FDC_GetDMAAddress();
 	count = HDC_GetCount(ctr);
 
-	LOG_TRACE(TRACE_SCSI_CMD, "HDC: INQUIRY (%s) to 0x%x.\n",
-	          HDC_CmdInfoStr(ctr), nDmaAddr);
+	LOG_TRACE(TRACE_SCSI_CMD, "HDC: INQUIRY (%s)", HDC_CmdInfoStr(ctr));
+
+	buf = HDC_PrepRespBuf(ctr, count);
+	memcpy(buf, inquiry_bytes, count);
 
 	if (count > (int)sizeof(inquiry_bytes))
 		count = sizeof(inquiry_bytes);
 
 	/* For unsupported LUNs set the Peripheral Qualifier and the
 	 * Peripheral Device Type according to the SCSI standard */
-	inquiry_bytes[0] = HDC_GetLUN(ctr) == 0 ? 0 : 0x7F;
+	buf[0] = HDC_GetLUN(ctr) == 0 ? 0 : 0x7F;
 
-	inquiry_bytes[4] = count - 5;
+	buf[4] = count - 5;
 
-	if (STMemory_SafeCopy(nDmaAddr, inquiry_bytes, count, "HDC DMA inquiry"))
-	{
-		ctr->returnCode = HD_STATUS_OK;
-	}
-	else
-	{
-		ctr->bDmaError = true;
-		ctr->returnCode = HD_STATUS_ERROR;
-	}
-
-	FDC_WriteDMAAddress(nDmaAddr + count);
+	ctr->returnCode = HD_STATUS_OK;
 
 	dev->nLastError = HD_REQSENS_OK;
 	dev->bSetLastBlockAddr = false;
@@ -231,9 +252,8 @@ static void HDC_Cmd_Inquiry(SCSI_CTRLR *ctr)
 static void HDC_Cmd_RequestSense(SCSI_CTRLR *ctr)
 {
 	SCSI_DEV *dev = &ctr->devs[ctr->target];
-	Uint32 nDmaAddr;
 	int nRetLen;
-	Uint8 retbuf[22];
+	Uint8 *retbuf;
 
 	nRetLen = HDC_GetCount(ctr);
 
@@ -254,8 +274,7 @@ static void HDC_Cmd_RequestSense(SCSI_CTRLR *ctr)
 		nRetLen = 22;
 	}
 
-	nDmaAddr = FDC_GetDMAAddress();
-
+	retbuf = HDC_PrepRespBuf(ctr, nRetLen);
 	memset(retbuf, 0, nRetLen);
 
 	if (nRetLen <= 4)
@@ -302,17 +321,7 @@ static void HDC_Cmd_RequestSense(SCSI_CTRLR *ctr)
 	fprintf(stderr,"\n");
 	*/
 
-	if (STMemory_SafeCopy(nDmaAddr, retbuf, nRetLen, "HDC request sense"))
-	{
-		ctr->returnCode = HD_STATUS_OK;
-	}
-	else
-	{
-		ctr->bDmaError = true;
-		ctr->returnCode = HD_STATUS_ERROR;
-	}
-
-	FDC_WriteDMAAddress(nDmaAddr + nRetLen);
+	ctr->returnCode = HD_STATUS_OK;
 }
 
 
@@ -323,57 +332,45 @@ static void HDC_Cmd_RequestSense(SCSI_CTRLR *ctr)
 static void HDC_Cmd_ModeSense(SCSI_CTRLR *ctr)
 {
 	SCSI_DEV *dev = &ctr->devs[ctr->target];
-	Uint32 nDmaAddr;
+	Uint8 *buf;
 
 	LOG_TRACE(TRACE_SCSI_CMD, "HDC: MODE SENSE (%s).\n", HDC_CmdInfoStr(ctr));
 
-	nDmaAddr = FDC_GetDMAAddress();
+	dev->bSetLastBlockAddr = false;
 
-	if (!STMemory_ValidArea(nDmaAddr, 16))
-	{
-		Log_Printf(LOG_WARN, "HCD mode sense uses invalid RAM range 0x%x+%i\n", nDmaAddr, 16);
-		ctr->bDmaError = true;
-		ctr->returnCode = HD_STATUS_ERROR;
-	}
-	else if (ctr->command[2] == 0 && HDC_GetCount(ctr) == 0x10)
-	{
-		size_t blocks;
-		blocks = dev->hdSize;
-
-		STRam[nDmaAddr+0] = 0;
-		STRam[nDmaAddr+1] = 0;
-		STRam[nDmaAddr+2] = 0;
-		STRam[nDmaAddr+3] = 8;
-		STRam[nDmaAddr+4] = 0;
-
-		STRam[nDmaAddr+5] = blocks >> 16;  // Number of blocks, high (?)
-		STRam[nDmaAddr+6] = blocks >> 8;   // Number of blocks, med (?)
-		STRam[nDmaAddr+7] = blocks;        // Number of blocks, low (?)
-
-		STRam[nDmaAddr+8] = 0;
-
-		STRam[nDmaAddr+9] = 0;      // Block size in bytes, high
-		STRam[nDmaAddr+10] = 2;     // Block size in bytes, med
-		STRam[nDmaAddr+11] = 0;     // Block size in bytes, low
-
-		STRam[nDmaAddr+12] = 0;
-		STRam[nDmaAddr+13] = 0;
-		STRam[nDmaAddr+14] = 0;
-		STRam[nDmaAddr+15] = 0;
-
-		FDC_WriteDMAAddress(nDmaAddr + 16);
-
-		ctr->returnCode = HD_STATUS_OK;
-		dev->nLastError = HD_REQSENS_OK;
-	}
-	else
+	if (ctr->command[2] != 0 || HDC_GetCount(ctr) != 0x10)
 	{
 		Log_Printf(LOG_TODO, "HDC: Unsupported MODE SENSE command\n");
 		ctr->returnCode = HD_STATUS_ERROR;
 		dev->nLastError = HD_REQSENS_INVARG;
+		return;
 	}
 
-	dev->bSetLastBlockAddr = false;
+	buf = HDC_PrepRespBuf(ctr, 16);
+
+	buf[0] = 0;
+	buf[1] = 0;
+	buf[2] = 0;
+	buf[3] = 8;
+	buf[4] = 0;
+
+	buf[5] = dev->hdSize >> 16;  // Number of blocks, high
+	buf[6] = dev->hdSize >> 8;   // Number of blocks, med
+	buf[7] = dev->hdSize;        // Number of blocks, low
+
+	buf[8] = 0;
+
+	buf[9] = 0;      // Block size in bytes, high
+	buf[10] = 2;     // Block size in bytes, med
+	buf[11] = 0;     // Block size in bytes, low
+
+	buf[12] = 0;
+	buf[13] = 0;
+	buf[14] = 0;
+	buf[15] = 0;
+
+	ctr->returnCode = HD_STATUS_OK;
+	dev->nLastError = HD_REQSENS_OK;
 }
 
 
@@ -400,38 +397,24 @@ static void HDC_Cmd_FormatDrive(SCSI_CTRLR *ctr)
 static void HDC_Cmd_ReadCapacity(SCSI_CTRLR *ctr)
 {
 	SCSI_DEV *dev = &ctr->devs[ctr->target];
-	Uint32 nDmaAddr = FDC_GetDMAAddress();
+	int nSectors = dev->hdSize - 1;
+	Uint8 *buf;
 
-	LOG_TRACE(TRACE_SCSI_CMD, "HDC: READ CAPACITY (%s) to 0x%x.\n",
-	          HDC_CmdInfoStr(ctr), nDmaAddr);
+	LOG_TRACE(TRACE_SCSI_CMD, "HDC: READ CAPACITY (%s)\n", HDC_CmdInfoStr(ctr));
 
-	/* seek to the position */
-	if (STMemory_ValidArea(nDmaAddr, 8))
-	{
-		int nSectors = dev->hdSize - 1;
-		STRam[nDmaAddr++] = (nSectors >> 24) & 0xFF;
-		STRam[nDmaAddr++] = (nSectors >> 16) & 0xFF;
-		STRam[nDmaAddr++] = (nSectors >> 8) & 0xFF;
-		STRam[nDmaAddr++] = (nSectors) & 0xFF;
-		STRam[nDmaAddr++] = 0x00;
-		STRam[nDmaAddr++] = 0x00;
-		STRam[nDmaAddr++] = 0x02;
-		STRam[nDmaAddr++] = 0x00;
+	buf = HDC_PrepRespBuf(ctr, 8);
 
-		/* Update DMA counter */
-		FDC_WriteDMAAddress(nDmaAddr);
-
-		ctr->returnCode = HD_STATUS_OK;
-		dev->nLastError = HD_REQSENS_OK;
-	}
-	else
-	{
-		Log_Printf(LOG_WARN, "HDC capacity read uses invalid RAM range 0x%x+%i\n", nDmaAddr, 8);
-		ctr->bDmaError = true;
-		ctr->returnCode = HD_STATUS_ERROR;
-		dev->nLastError = HD_REQSENS_NOSECTOR;
-	}
+	buf[0] = (nSectors >> 24) & 0xFF;
+	buf[1] = (nSectors >> 16) & 0xFF;
+	buf[2] = (nSectors >> 8) & 0xFF;
+	buf[3] = (nSectors) & 0xFF;
+	buf[4] = 0x00;
+	buf[5] = 0x00;
+	buf[6] = 0x02;
+	buf[7] = 0x00;
 
+	ctr->returnCode = HD_STATUS_OK;
+	dev->nLastError = HD_REQSENS_OK;
 	dev->bSetLastBlockAddr = false;
 }
 
@@ -447,7 +430,7 @@ static void HDC_Cmd_WriteSector(SCSI_CTRLR *ctr)
 
 	dev->nLastBlockAddr = HDC_GetLBA(ctr);
 
-	LOG_TRACE(TRACE_SCSI_CMD, "HDC: WRITE SECTOR (%s) with LBA 0x%x from 0x%x.\n",
+	LOG_TRACE(TRACE_SCSI_CMD, "HDC: WRITE SECTOR (%s) with LBA 0x%x from 0x%x",
 	          HDC_CmdInfoStr(ctr), dev->nLastBlockAddr, nDmaAddr);
 
 	/* seek to the position */
@@ -461,7 +444,7 @@ static void HDC_Cmd_WriteSector(SCSI_CTRLR *ctr)
 	{
 		/* write - if allowed */
 #ifndef DISALLOW_HDC_WRITE
-		if (STMemory_ValidArea(nDmaAddr, 512 * HDC_GetCount(ctr)))
+		if ( STMemory_CheckAreaType ( nDmaAddr , 512 * HDC_GetCount(ctr) , ABFLAG_RAM ) )
 		{
 			n = fwrite(&STRam[nDmaAddr], 512,
 				   HDC_GetCount(ctr), dev->image_file);
@@ -487,6 +470,9 @@ static void HDC_Cmd_WriteSector(SCSI_CTRLR *ctr)
 		/* Update DMA counter */
 		FDC_WriteDMAAddress(nDmaAddr + 512*n);
 	}
+	LOG_TRACE(TRACE_SCSI_CMD, " -> %s (%d)\n",
+		  ctr->returnCode == HD_STATUS_OK ? "OK" : "ERROR",
+		  dev->nLastError);
 
 	dev->bSetLastBlockAddr = true;
 }
@@ -498,13 +484,13 @@ static void HDC_Cmd_WriteSector(SCSI_CTRLR *ctr)
 static void HDC_Cmd_ReadSector(SCSI_CTRLR *ctr)
 {
 	SCSI_DEV *dev = &ctr->devs[ctr->target];
-	Uint32 nDmaAddr = FDC_GetDMAAddress();
+	Uint8 *buf;
 	int n;
 
 	dev->nLastBlockAddr = HDC_GetLBA(ctr);
 
-	LOG_TRACE(TRACE_SCSI_CMD, "HDC: READ SECTOR (%s) with LBA 0x%x to 0x%x.\n",
-	          HDC_CmdInfoStr(ctr), dev->nLastBlockAddr, nDmaAddr);
+	LOG_TRACE(TRACE_SCSI_CMD, "HDC: READ SECTOR (%s) with LBA 0x%x",
+	          HDC_CmdInfoStr(ctr), dev->nLastBlockAddr);
 
 	/* seek to the position */
 	if (dev->nLastBlockAddr >= dev->hdSize ||
@@ -515,18 +501,8 @@ static void HDC_Cmd_ReadSector(SCSI_CTRLR *ctr)
 	}
 	else
 	{
-		if (STMemory_ValidArea(nDmaAddr, 512 * HDC_GetCount(ctr)))
-		{
-			n = fread(&STRam[nDmaAddr], 512,
-				   HDC_GetCount(ctr), dev->image_file);
-		}
-		else
-		{
-			Log_Printf(LOG_WARN, "HDC sector read uses invalid RAM range 0x%x+%i\n",
-				   nDmaAddr, 512 * HDC_GetCount(ctr));
-			ctr->bDmaError = true;
-			n = 0;
-		}
+		buf = HDC_PrepRespBuf(ctr, 512 * HDC_GetCount(ctr));
+		n = fread(buf, 512, HDC_GetCount(ctr), dev->image_file);
 		if (n == HDC_GetCount(ctr))
 		{
 			ctr->returnCode = HD_STATUS_OK;
@@ -537,10 +513,10 @@ static void HDC_Cmd_ReadSector(SCSI_CTRLR *ctr)
 			ctr->returnCode = HD_STATUS_ERROR;
 			dev->nLastError = HD_REQSENS_NOSECTOR;
 		}
-
-		/* Update DMA counter */
-		FDC_WriteDMAAddress(nDmaAddr + 512*n);
 	}
+	LOG_TRACE(TRACE_SCSI_CMD, " -> %s (%d)\n",
+		  ctr->returnCode == HD_STATUS_OK ? "OK" : "ERROR",
+		  dev->nLastError);
 
 	dev->bSetLastBlockAddr = true;
 }
@@ -563,6 +539,8 @@ static void HDC_EmulateCommandPacket(SCSI_CTRLR *ctr)
 {
 	SCSI_DEV *dev = &ctr->devs[ctr->target];
 
+	ctr->respcnt = 0;
+
 	switch (ctr->opcode)
 	{
 	 case HD_TEST_UNIT_RDY:
@@ -635,97 +613,190 @@ static void HDC_EmulateCommandPacket(SCSI_CTRLR *ctr)
 
 /*---------------------------------------------------------------------*/
 /**
- * Print data about the hard drive image
+ * Return given image file (primary) partition count and with tracing
+ * print also partition table.
+ *
+ * Supports both DOS and Atari master boot record partition tables
+ * (with 4 entries).
+ *
+ * TODO:
+ * - Support also Atari ICD (12 entries, at offset 0x156) and
+ *   extended partition schemes.  Linux kernel has code for those:
+ *   http://lxr.free-electrons.com/source/block/partitions/atari.c
+ * - Support partition tables with other endianess
  */
-static void HDC_GetInfo(SCSI_DEV *dev)
+int HDC_PartitionCount(FILE *fp, const Uint64 tracelevel)
 {
-/* Partition table contains hd size + 4 partition entries
- * (composed of flag byte, 3 char ID, start offset and size),
- * this is followed by bad sector list + count and the root sector checksum.
- * Before this there's the boot code and with ICD hd driver additional 8
- * partition entries (at offset 0x156).
- */
-#define HD_PARTITIONTABLE_SIZE (4+4*12)
-#define HD_PARTITIONTABLE_OFFSET 0x1C2
-	long offset;
-	unsigned char hdinfo[HD_PARTITIONTABLE_SIZE];
-	int i;
+	unsigned char *pinfo, bootsector[512];
+	Uint32 start, sectors, total = 0;
+	int i, parts = 0;
+	off_t offset;
 
-	if (dev->image_file == NULL)
-		return;
-	offset = ftell(dev->image_file);
+	if (!fp)
+		return 0;
+	offset = ftello(fp);
 
-	fseek(dev->image_file, HD_PARTITIONTABLE_OFFSET, 0);
-	if (fread(hdinfo, HD_PARTITIONTABLE_SIZE, 1, dev->image_file) != 1)
+	if (fseeko(fp, 0, SEEK_SET) != 0
+	    || fread(bootsector, sizeof(bootsector), 1, fp) != 1)
 	{
-		perror("HDC_GetInfo");
-		return;
+		perror("HDC_PartitionCount");
+		return 0;
 	}
 
-#ifdef HDC_VERBOSE
-	fprintf(stderr, "Total disk size according to MBR: %i Mb\n",
-		HDC_ReadInt32(hdinfo, 0) >> 11);
-	/* flags for each partition entry are zero if they are not valid */
-	fprintf(stderr, "Partition 0 exists?: %s\n", (hdinfo[4] != 0)?"Yes":"No");
-	fprintf(stderr, "Partition 1 exists?: %s\n", (hdinfo[4+12] != 0)?"Yes":"No");
-	fprintf(stderr, "Partition 2 exists?: %s\n", (hdinfo[4+24] != 0)?"Yes":"No");
-	fprintf(stderr, "Partition 3 exists?: %s\n", (hdinfo[4+36] != 0)?"Yes":"No");
-#endif
+	if (bootsector[0x1FE] == 0x55 && bootsector[0x1FF] == 0xAA)
+	{
+		int ptype, boot;
 
-	for(i=0;i<4;i++)
-		if(hdinfo[4 + 12*i])
-			nPartitions++;
+		LOG_TRACE(tracelevel, "DOS MBR:\n");
+		/* first partition table entry */
+		pinfo = bootsector + 0x1BE;
+		for (i = 0; i < 4; i++, pinfo += 16)
+		{
+			boot = pinfo[0];
+			ptype = pinfo[4];
+			start = HDC_ReadInt32(pinfo, 8);
+			sectors = HDC_ReadInt32(pinfo, 12);
+			total += sectors;
+			LOG_TRACE(tracelevel, "- Partition %d: type=0x%02x, start=0x%08x, size=%d MB %s\n",
+				  i, ptype, start, sectors/2048, boot ? "(boot)" : "");
+			if (ptype)
+				parts++;
+		}
+		LOG_TRACE(tracelevel, "- Total size: %i MB in %d partitions\n", total/2048, parts);
+	}
+	else
+	{
+		/* Partition table contains hd size + 4 partition entries
+		 * (composed of flag byte, 3 char ID, start offset
+		 * and size), this is followed by bad sector list +
+		 * count and the root sector checksum. Before this
+		 * there's the boot code.
+		 */
+		char c, pid[4];
+		int j, flags;
 
-	fseek(dev->image_file, offset, 0);
+		LOG_TRACE(tracelevel, "ATARI MBR:\n");
+		pinfo = bootsector + 0x1C6;
+		for (i = 0; i < 4; i++, pinfo += 12)
+		{
+			flags = pinfo[0];
+			for (j = 0; j < 3; j++)
+			{
+				c = pinfo[j+1];
+				if (c < 32 || c >= 127)
+					c = '.';
+				pid[j] = c;
+			}
+			pid[3] = '\0';
+			start = HDC_ReadInt32(pinfo, 4);
+			sectors = HDC_ReadInt32(pinfo, 8);
+			LOG_TRACE(tracelevel, "- Partition %d: ID=%s, start=0x%08x, size=%d MB, flags=0x%x\n",
+				  i, pid, start, sectors/2048, flags);
+			if (flags & 0x1)
+				parts++;
+		}
+		total = HDC_ReadInt32(bootsector, 0x1C2);
+		LOG_TRACE(tracelevel, "- Total size: %i MB in %d partitions\n", total/2048, parts);
+	}
+
+	if (fseeko(fp, offset, SEEK_SET) != 0)
+		perror("HDC_PartitionCount");
+	return parts;
 }
 
+/**
+ * Open a disk image file
+ */
+static int HDC_InitDevice(SCSI_DEV *dev, char *filename)
+{
+	off_t filesize;
+	FILE *fp;
+
+	dev->enabled = false;
+	Log_Printf(LOG_INFO, "Mounting hard drive image '%s'\n", filename);
+
+	/* Check size for sanity - is the length a multiple of 512? */
+	filesize = File_Length(filename);
+	if (filesize < 0)
+	{
+		Log_Printf(LOG_ERROR, "ERROR: unable to access/get HD file size!\n");
+		return -EINVAL;
+	}
+	if (filesize <= 0 || (filesize & 0x1ff) != 0)
+	{
+		Log_Printf(LOG_ERROR, "ERROR: HD file has strange size!\n");
+		return -EINVAL;
+	}
+
+	fp = fopen(filename, "rb+");
+	if (fp == NULL)
+	{
+		Log_Printf(LOG_ERROR, "ERROR: cannot open HD file read/write!\n");
+		return -ENOENT;
+	}
+	if (!File_Lock(fp))
+	{
+		Log_Printf(LOG_ERROR, "ERROR: cannot lock HD file for writing!\n");
+		fclose(fp);
+		return -ENOLCK;
+	}
+
+	dev->hdSize = filesize / 512;
+	dev->image_file = fp;
+	dev->enabled = true;
+
+	return 0;
+}
 
 /**
  * Open the disk image file, set partitions.
  */
 bool HDC_Init(void)
 {
-	off_t filesize;
 	int i;
 
-	memset(&AcsiBus, 0, sizeof(AcsiBus));
-	nPartitions = 0;
+	/* ACSI */
+	nAcsiPartitions = 0;
 	bAcsiEmuOn = false;
-
+	memset(&AcsiBus, 0, sizeof(AcsiBus));
+	AcsiBus.respbufsize = 512;
+	AcsiBus.resp = malloc(AcsiBus.respbufsize);
+	if (!AcsiBus.resp)
+	{
+		perror("HDC_Init");
+		return false;
+	}
 	for (i = 0; i < MAX_ACSI_DEVS; i++)
 	{
-		char *filename;
-
 		if (!ConfigureParams.Acsi[i].bUseDevice)
 			continue;
-		filename = ConfigureParams.Acsi[i].sDeviceFile;
-
-		/* Check size for sanity - is the length a multiple of 512? */
-		filesize = File_Length(filename);
-		if (filesize <= 0 || (filesize & 0x1ff) != 0)
+		if (HDC_InitDevice(&AcsiBus.devs[i], ConfigureParams.Acsi[i].sDeviceFile) == 0)
 		{
-			Log_Printf(LOG_ERROR, "HD file '%s' has strange size!\n",
-			           filename);
-			continue;
+			bAcsiEmuOn = true;
+			nAcsiPartitions += HDC_PartitionCount(AcsiBus.devs[i].image_file, TRACE_SCSI_CMD);
 		}
-		AcsiBus.devs[i].hdSize = filesize / 512;
-
-		AcsiBus.devs[i].image_file = fopen(filename, "rb+");
-		if (AcsiBus.devs[i].image_file == NULL)
-		{
-			Log_Printf(LOG_ERROR, "Can not open HD file '%s'!\n",
-			           filename);
-			continue;
-		}
-
-		HDC_GetInfo(&AcsiBus.devs[i]);
-		Log_Printf(LOG_INFO, "Hard drive image %s mounted.\n", filename);
-		AcsiBus.devs[i].enabled = true;
-		bAcsiEmuOn = true;
 	}
 
+	/* SCSI */
+#if WITH_NCR5380
+	memset(&ScsiBus, 0, sizeof(ScsiBus));
+	ScsiBus.respbufsize = 512;
+	ScsiBus.resp = malloc(ScsiBus.respbufsize);
+	if (!ScsiBus.resp)
+	{
+		perror("HDC_Init");
+		return bAcsiEmuOn;
+	}
+	for (i = 0; i < MAX_SCSI_DEVS; i++)
+	{
+		if (!ConfigureParams.Scsi[i].bUseDevice)
+			continue;
+		HDC_InitDevice(&ScsiBus.devs[i], ConfigureParams.Scsi[i].sDeviceFile);
+	}
+#endif
+
 	/* set number of partitions */
-	nNumDrives += nPartitions;
+	nNumDrives += nAcsiPartitions;
 
 	return bAcsiEmuOn;
 }
@@ -747,13 +818,30 @@ void HDC_UnInit(void)
 	{
 		if (!AcsiBus.devs[i].enabled)
 			continue;
+		File_UnLock(AcsiBus.devs[i].image_file);
 		fclose(AcsiBus.devs[i].image_file);
 		AcsiBus.devs[i].image_file = NULL;
 		AcsiBus.devs[i].enabled = false;
 	}
+	free(AcsiBus.resp);
+	AcsiBus.resp = NULL;
 
-	nNumDrives -= nPartitions;
-	nPartitions = 0;
+#if WITH_NCR5380
+	for (i = 0; i < MAX_SCSI_DEVS; i++)
+	{
+		if (!ScsiBus.devs[i].enabled)
+			continue;
+		File_UnLock(ScsiBus.devs[i].image_file);
+		fclose(ScsiBus.devs[i].image_file);
+		ScsiBus.devs[i].image_file = NULL;
+		ScsiBus.devs[i].enabled = false;
+	}
+	free(ScsiBus.resp);
+	ScsiBus.resp = NULL;
+#endif
+
+	nNumDrives -= nAcsiPartitions;
+	nAcsiPartitions = 0;
 	bAcsiEmuOn = false;
 }
 
@@ -771,16 +859,18 @@ void HDC_ResetCommandStatus(void)
 
 /**
  * Process HDC command packets (SCSI/ACSI) bytes.
+ * @return true if command has been executed.
  */
-static void HDC_WriteCommandPacket(SCSI_CTRLR *ctr, Uint8 b)
+static bool HDC_WriteCommandPacket(SCSI_CTRLR *ctr, Uint8 b)
 {
+	bool bDidCmd = false;
 	SCSI_DEV *dev = &ctr->devs[ctr->target];
 
 	/* Abort if the target device is not enabled */
 	if (!dev->enabled)
 	{
 		ctr->returnCode = HD_STATUS_ERROR;
-		return;
+		return false;
 	}
 
 	/* Extract ACSI/SCSI opcode */
@@ -804,6 +894,7 @@ static void HDC_WriteCommandPacket(SCSI_CTRLR *ctr, Uint8 b)
 		if (HDC_GetLUN(ctr) == 0 || ctr->opcode == HD_INQUIRY)
 		{
 			HDC_EmulateCommandPacket(ctr);
+			bDidCmd = true;
 		}
 		else
 		{
@@ -812,9 +903,14 @@ static void HDC_WriteCommandPacket(SCSI_CTRLR *ctr, Uint8 b)
 			dev->nLastError = HD_REQSENS_INVLUN;
 			/* REQUEST SENSE is still handled for invalid LUNs */
 			if (ctr->opcode == HD_REQ_SENSE)
+			{
 				HDC_Cmd_RequestSense(ctr);
+				bDidCmd = true;
+			}
 			else
+			{
 				ctr->returnCode = HD_STATUS_ERROR;
+			}
 		}
 
 		ctr->byteCount = 0;
@@ -835,6 +931,8 @@ static void HDC_WriteCommandPacket(SCSI_CTRLR *ctr, Uint8 b)
 	{
 		ctr->returnCode = HD_STATUS_OK;
 	}
+
+	return bDidCmd;
 }
 
 /*---------------------------------------------------------------------*/
@@ -859,6 +957,7 @@ void Ncr5380_Reset(void)
  */
 static void Ncr5380_WriteByte(int addr, Uint8 byte)
 {
+#if WITH_NCR5380
 	switch (addr)
 	{
 	case 0:			/* Output Data register */
@@ -882,6 +981,7 @@ static void Ncr5380_WriteByte(int addr, Uint8 byte)
 	default:
 		fprintf(stderr, "Unexpected NCR5380 address\n");
 	}
+#endif
 }
 
 /**
@@ -889,6 +989,7 @@ static void Ncr5380_WriteByte(int addr, Uint8 byte)
  */
 static Uint8 Ncr5380_ReadByte(int addr)
 {
+#if WITH_NCR5380
 	switch (addr)
 	{
 	case 0:			/* Current SCSI Data register */
@@ -924,6 +1025,7 @@ static Uint8 Ncr5380_ReadByte(int addr)
 	default:
 		fprintf(stderr, "Unexpected NCR5380 address\n");
 	}
+#endif
 
 	return 0;
 }
@@ -931,8 +1033,14 @@ static Uint8 Ncr5380_ReadByte(int addr)
 
 static void Acsi_WriteCommandByte(int addr, Uint8 byte)
 {
-	/* When the A1 pin is pushed to 0, we want to start a new command */
-	if ((addr & 2) == 0)
+	/* Clear IRQ initially (will be set again if byte has been accepted) */
+	FDC_ClearHdcIRQ();
+
+	/* When the A1 pin is pushed to 0, we want to start a new command.
+	 * We ignore the pin for the second byte in the packet since this
+	 * seems to happen on real hardware too (some buggy driver relies
+	 * on this behavior). */
+	if ((addr & 2) == 0 && AcsiBus.byteCount != 1)
 	{
 		AcsiBus.byteCount = 0;
 		AcsiBus.target = ((byte & 0xE0) >> 5);
@@ -951,7 +1059,19 @@ static void Acsi_WriteCommandByte(int addr, Uint8 byte)
 	else
 	{
 		/* Process normal command byte */
-		HDC_WriteCommandPacket(&AcsiBus, byte);
+		bool bDidCmd = HDC_WriteCommandPacket(&AcsiBus, byte);
+		if (bDidCmd && AcsiBus.returnCode == HD_STATUS_OK && AcsiBus.respcnt)
+		{
+			/* DMA transfer necessary */
+			Uint32 nDmaAddr = FDC_GetDMAAddress();
+			if (!STMemory_SafeCopy(nDmaAddr, AcsiBus.resp, AcsiBus.respcnt, "ACSI DMA"))
+			{
+				AcsiBus.bDmaError = true;
+				AcsiBus.returnCode = HD_STATUS_ERROR;
+			}
+			FDC_WriteDMAAddress(nDmaAddr + AcsiBus.respcnt);
+			AcsiBus.respcnt = 0;
+		}
 	}
 
 	if (AcsiBus.devs[AcsiBus.target].enabled)
@@ -959,11 +1079,6 @@ static void Acsi_WriteCommandByte(int addr, Uint8 byte)
 		FDC_SetDMAStatus(AcsiBus.bDmaError);	/* Mark DMA error */
 		FDC_SetIRQ(FDC_IRQ_SOURCE_HDC);
 	}
-	else
-	{
-		/* If there's no target, the interrupt line stays high */
-		MFP_GPIP |= 0x20;
-	}
 }
 
 /**
