@@ -18,27 +18,21 @@
 #include "configuration.h"
 #include "file.h"
 #include "ide.h"
+#include "hdc.h" /* for partition counting */
 #include "m68000.h"
 #include "mfp.h"
 #include "stMemory.h"
+#include "str.h"
 #include "sysdeps.h"
 
 #if HAVE_MALLOC_H
 # include <malloc.h>
 #endif
 
-
+int nIDEPartitions = 0;
 struct IDEState;
 
 
-#define IDE_DEBUG 0
-
-#if IDE_DEBUG
-#define Dprintf(a) printf a
-#else
-#define Dprintf(a)
-#endif
-
 static struct IDEState *opaque_ide_if;
 
 static void ide_ioport_write(void *opaque, uint32_t addr, uint32_t val);
@@ -91,14 +85,12 @@ uae_u32 Ide_Mem_bget(uaecptr addr)
 	int ideport;
 	uint8_t retval;
 
-	Dprintf(("IdeMem_bget($%x)\n", addr));
-
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
 	if (addr >= 0xf00040 || !ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, BUS_ERROR_READ);
+		M68000_BusError(addr, BUS_ERROR_READ, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
 		return -1;
 	}
 
@@ -117,6 +109,7 @@ uae_u32 Ide_Mem_bget(uaecptr addr)
 		retval = 0xFF;
 	}
 
+	LOG_TRACE(TRACE_IDE, "IDE: bget($%x) = $%02x\n", addr, retval);
 	return retval;
 }
 
@@ -133,9 +126,7 @@ uae_u32 Ide_Mem_wget(uaecptr addr)
 	if (addr >= 0xf00040 || !ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, BUS_ERROR_READ);
-		if (ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage)
-			fprintf(stderr, "Illegal IDE IO memory access: IdeMem_wget($%x)\n", addr);
+		M68000_BusError(addr, BUS_ERROR_READ, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
 		return -1;
 	}
 
@@ -148,9 +139,7 @@ uae_u32 Ide_Mem_wget(uaecptr addr)
 		retval = 0xFFFF;
 	}
 
-	Dprintf(("IdeMem_wget($%x) = $%04x\n", addr, retval));
-
-
+	LOG_TRACE(TRACE_IDE, "IDE: wget($%x) = $%04x\n", addr, retval);
 	return retval;
 }
 
@@ -167,9 +156,7 @@ uae_u32 Ide_Mem_lget(uaecptr addr)
 	if (addr >= 0xf00040 || !ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, BUS_ERROR_READ);
-		if (ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage)
-			fprintf(stderr, "Illegal IDE IO memory access: IdeMem_lget($%x)\n", addr);
+		M68000_BusError(addr, BUS_ERROR_READ, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
 		return -1;
 	}
 
@@ -185,8 +172,7 @@ uae_u32 Ide_Mem_lget(uaecptr addr)
 	/* word swap for long access to data register */
 	retval = ((retval >> 16) & 0x0000ffff) | ((retval & 0x0000ffff) << 16);
 
-	Dprintf(("IdeMem_lget($%x) = $%08x\n", addr, retval));
-
+	LOG_TRACE(TRACE_IDE, "IDE: lget($%x) = $%08x\n", addr, retval);
 	return retval;
 }
 
@@ -201,13 +187,12 @@ void Ide_Mem_bput(uaecptr addr, uae_u32 val)
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 	val &= 0x0ff;
 
-	Dprintf(("IdeMem_bput($%x, $%x)\n", addr, val));
+	LOG_TRACE(TRACE_IDE, "IDE: bput($%x, $%x)\n", addr, val);
 
 	if (addr >= 0xf00040 || !ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, BUS_ERROR_WRITE);
-		//fprintf(stderr, "Illegal IDE IO memory access: IdeMem_bput($%x)\n", addr);
+		M68000_BusError(addr, BUS_ERROR_WRITE, BUS_ERROR_SIZE_BYTE, BUS_ERROR_ACCESS_DATA);
 		return;
 	}
 
@@ -232,13 +217,12 @@ void Ide_Mem_wput(uaecptr addr, uae_u32 val)
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 	val &= 0x0ffff;
 
-	Dprintf(("IdeMem_wput($%x, $%x)\n", addr, val));
+	LOG_TRACE(TRACE_IDE, "IDE: wput($%x, $%x)\n", addr, val);
 
 	if (addr >= 0xf00040 || !ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, BUS_ERROR_WRITE);
-		//fprintf(stderr, "Illegal IDE IO memory access: IdeMem_wput($%x)\n", addr);
+		M68000_BusError(addr, BUS_ERROR_WRITE, BUS_ERROR_SIZE_WORD, BUS_ERROR_ACCESS_DATA);
 		return;
 	}
 
@@ -256,13 +240,12 @@ void Ide_Mem_lput(uaecptr addr, uae_u32 val)
 {
 	addr &= 0x00ffffff;                           /* Use a 24 bit address */
 
-	Dprintf(("IdeMem_lput($%x, $%x)\n", addr, val));
+	LOG_TRACE(TRACE_IDE, "IDE: lput($%x, $%x)\n", addr, val);
 
 	if (addr >= 0xf00040 || !ConfigureParams.HardDisk.bUseIdeMasterHardDiskImage)
 	{
 		/* invalid memory addressing --> bus error */
-		M68000_BusError(addr, BUS_ERROR_WRITE);
-		//fprintf(stderr, "Illegal IDE IO memory access: IdeMem_lput($%x)\n", addr);
+		M68000_BusError(addr, BUS_ERROR_WRITE, BUS_ERROR_SIZE_LONG, BUS_ERROR_ACCESS_DATA);
 		return;
 	}
 
@@ -506,11 +489,15 @@ static int bdrv_read(BlockDriverState *bs, int64_t sector_num,
 
 	len = nb_sectors * 512;
 
-	fseeko(bs->fhndl, sector_num*512, SEEK_SET);
+	if (fseeko(bs->fhndl, sector_num*512, SEEK_SET) != 0)
+	{
+		perror("bdrv_read");
+		return -errno;
+	}
 	ret = fread(buf, 1, len, bs->fhndl);
 	if (ret != len)
 	{
-		fprintf(stderr,"IDE bdrv_read error: (%d != %d length) at sector %lu!\n", ret, len, (unsigned long)sector_num);
+		fprintf(stderr,"IDE: bdrv_read error (%d != %d length) at sector %lu!\n", ret, len, (unsigned long)sector_num);
 		return -EINVAL;
 	}
 	else
@@ -540,11 +527,15 @@ static int bdrv_write(BlockDriverState *bs, int64_t sector_num,
 
 	len = nb_sectors * 512;
 
-	fseeko(bs->fhndl, sector_num*512, SEEK_SET);
+	if (fseeko(bs->fhndl, sector_num*512, SEEK_SET) != 0)
+	{
+		perror("bdrv_write");
+		return -errno;
+	}
 	ret = fwrite(buf, 1, len, bs->fhndl);
 	if (ret != len)
 	{
-		fprintf(stderr,"IDE bdrv_write error: (%d != %d length) at sector %lu!\n", ret, len,  (unsigned long)sector_num);
+		fprintf(stderr,"IDE: bdrv_write error (%d != %d length) at sector %lu!\n", ret, len,  (unsigned long)sector_num);
 		return -EIO;
 	}
 	else
@@ -558,9 +549,9 @@ static int bdrv_write(BlockDriverState *bs, int64_t sector_num,
 
 static int bdrv_open(BlockDriverState *bs, const char *filename, int flags)
 {
-	fprintf(stderr,"IDE: Opening %s\n", filename);
+	Log_Printf(LOG_INFO, "Mounting IDE hard drive image %s\n", filename);
 
-	strncpy(bs->filename, filename, sizeof(bs->filename));
+	strlcpy(bs->filename, filename, sizeof(bs->filename));
 
 	bs->read_only = 0;
 
@@ -573,6 +564,12 @@ static int bdrv_open(BlockDriverState *bs, const char *filename, int flags)
 			perror("bdrv_open");
 		bs->read_only = 1;
 	}
+	else if (!File_Lock(bs->fhndl))
+	{
+		Log_Printf(LOG_ERROR, "ERROR: cannot lock HD file for writing!\n");
+		fclose(bs->fhndl);
+		bs->fhndl = NULL;
+	}
 
 	/* call the change callback */
 	bs->media_changed = 1;
@@ -589,6 +586,7 @@ static void bdrv_flush(BlockDriverState *bs)
 
 static void bdrv_close(BlockDriverState *bs)
 {
+	File_UnLock(bs->fhndl);
 	fclose(bs->fhndl);
 	bs->fhndl = NULL;
 }
@@ -603,10 +601,6 @@ static void bdrv_eject(BlockDriverState *bs, int eject_flag)
 }
 
 
-/* debug IDE devices */
-// #define DEBUG_IDE
-// #define DEBUG_IDE_ATAPI
-
 // #define USE_DMA_CDROM
 
 /* Bits of HD_STATUS */
@@ -1121,9 +1115,8 @@ static inline void ide_set_irq(IDEState *s)
 {
 	if (!(s->cmd & IDE_CMD_DISABLE_IRQ))
 	{
-		/* raise IRQ */
-		MFP_InputOnChannel ( MFP_INT_FDCHDC , 0 );
-		MFP_GPIP &= ~0x20;
+		/* Set IRQ (set line to low) */
+		MFP_GPIP_Set_Line_Input ( MFP_GPIP_LINE_FDC_HDC , MFP_GPIP_STATE_LOW );
 	}
 }
 
@@ -1223,9 +1216,8 @@ static void ide_sector_read(IDEState *s)
 	}
 	else
 	{
-#if defined(DEBUG_IDE)
-		printf("IDE read sector=%Ld\n", sector_num);
-#endif
+		LOG_TRACE(TRACE_IDE, "IDE: read sector=%"PRId64"\n", sector_num);
+
 		if (n > s->req_nb_sectors)
 			n = s->req_nb_sectors;
 		ret = bdrv_read(s->bs, sector_num, s->io_buffer, n);
@@ -1250,9 +1242,8 @@ static void ide_sector_write(IDEState *s)
 
 	s->status = READY_STAT | SEEK_STAT;
 	sector_num = ide_get_sector(s);
-#if defined(DEBUG_IDE)
-	printf("IDE write sector=%Ld\n", sector_num);
-#endif
+	LOG_TRACE(TRACE_IDE, "IDE: write sector=%"PRId64"\n", sector_num);
+
 	n = s->nsector;
 	if (n > s->req_nb_sectors)
 		n = s->req_nb_sectors;
@@ -1292,9 +1283,8 @@ static void ide_atapi_cmd_ok(IDEState *s)
 
 static void ide_atapi_cmd_error(IDEState *s, int sense_key, int asc)
 {
-#ifdef DEBUG_IDE_ATAPI
-	printf("IDE atapi_cmd_error: sense=0x%x asc=0x%x\n", sense_key, asc);
-#endif
+	LOG_TRACE(TRACE_IDE, "IDE: ATAPI cmd error sense=0x%x asc=0x%x\n", sense_key, asc);
+
 	s->error = sense_key << 4;
 	s->status = READY_STAT | ERR_STAT;
 	s->nsector = (s->nsector & ~7) | ATAPI_INT_REASON_IO | ATAPI_INT_REASON_CD;
@@ -1394,12 +1384,12 @@ static void ide_atapi_io_error(IDEState *s, int ret)
 static void ide_atapi_cmd_reply_end(IDEState *s)
 {
 	int byte_count_limit, size, ret;
-#ifdef DEBUG_IDE_ATAPI
-	printf("IDE reply: tx_size=%d elem_tx_size=%d index=%d\n",
+
+	LOG_TRACE(TRACE_IDE, "IDE: ATAPI reply tx_size=%d elem_tx_size=%d index=%d\n",
 	       s->packet_transfer_size,
 	       s->elementary_transfer_size,
 	       s->io_buffer_index);
-#endif
+
 	if (s->packet_transfer_size <= 0)
 	{
 		/* end of transfer */
@@ -1407,9 +1397,7 @@ static void ide_atapi_cmd_reply_end(IDEState *s)
 		s->status = READY_STAT;
 		s->nsector = (s->nsector & ~7) | ATAPI_INT_REASON_IO | ATAPI_INT_REASON_CD;
 		ide_set_irq(s);
-#ifdef DEBUG_IDE_ATAPI
-		printf("IDE status=0x%x\n", s->status);
-#endif
+		LOG_TRACE(TRACE_IDE, "IDE: ATAPI status=0x%x\n", s->status);
 	}
 	else
 	{
@@ -1444,9 +1432,8 @@ static void ide_atapi_cmd_reply_end(IDEState *s)
 			/* a new transfer is needed */
 			s->nsector = (s->nsector & ~7) | ATAPI_INT_REASON_IO;
 			byte_count_limit = s->lcyl | (s->hcyl << 8);
-#ifdef DEBUG_IDE_ATAPI
-			printf("IDE byte_count_limit=%d\n", byte_count_limit);
-#endif
+			LOG_TRACE(TRACE_IDE, "IDE: ATAPI byte_count_limit=%d\n", byte_count_limit);
+
 			if (byte_count_limit == 0xffff)
 				byte_count_limit--;
 			size = s->packet_transfer_size;
@@ -1472,9 +1459,8 @@ static void ide_atapi_cmd_reply_end(IDEState *s)
 			s->elementary_transfer_size -= size;
 			s->io_buffer_index += size;
 			ide_set_irq(s);
-#ifdef DEBUG_IDE_ATAPI
-			printf("IDE status=0x%x\n", s->status);
-#endif
+
+			LOG_TRACE(TRACE_IDE, "IDE: ATAPI status=0x%x\n", s->status);
 		}
 	}
 }
@@ -1498,9 +1484,8 @@ static void ide_atapi_cmd_reply(IDEState *s, int size, int max_size)
 static void ide_atapi_cmd_read(IDEState *s, int lba, int nb_sectors,
                                int sector_size)
 {
-#ifdef DEBUG_IDE_ATAPI
-	printf("IDE read pio: LBA=%d nb_sectors=%d\n", lba, nb_sectors);
-#endif
+	LOG_TRACE(TRACE_IDE, "IDE: ATAPI read pio LBA=%d nb_sectors=%d\n", lba, nb_sectors);
+
 	s->lba = lba;
 	s->packet_transfer_size = nb_sectors * sector_size;
 	s->elementary_transfer_size = 0;
@@ -1520,17 +1505,17 @@ static void ide_atapi_cmd(IDEState *s)
 
 	packet = s->io_buffer;
 	buf = s->io_buffer;
-#ifdef DEBUG_IDE_ATAPI
+	if (LOG_TRACE_LEVEL(TRACE_IDE))
 	{
 		int i;
-		printf("IDE ATAPI limit=0x%x packet:", s->lcyl | (s->hcyl << 8));
+		LOG_TRACE_PRINT("IDE: ATAPI limit=0x%x packet", s->lcyl | (s->hcyl << 8));
 		for (i = 0; i < ATAPI_PACKET_SIZE; i++)
 		{
 			printf(" %02x", packet[i]);
 		}
 		printf("\n");
 	}
-#endif
+
 	switch (s->io_buffer[0])
 	{
 	case GPCMD_TEST_UNIT_READY:
@@ -1972,9 +1957,7 @@ static void ide_ioport_write(void *opaque, uint32_t addr, uint32_t val)
 	int unit, n;
 	int lba48 = 0;
 
-#ifdef DEBUG_IDE
-	printf("IDE: write addr=0x%x val=0x%02x\n", addr, val);
-#endif
+	LOG_TRACE(TRACE_IDE, "IDE: write addr=0x%x val=0x%02x\n", addr, val);
 
 	addr &= 7;
 	switch (addr)
@@ -2029,9 +2012,8 @@ static void ide_ioport_write(void *opaque, uint32_t addr, uint32_t val)
 	default:
 	case 7:
 		/* command */
-#if defined(DEBUG_IDE)
-		printf("IDE: CMD=%02x\n", val);
-#endif
+		LOG_TRACE(TRACE_IDE, "IDE: CMD=%02x\n", val);
+
 		s = ide_if->cur_drive;
 		/* ignore commands to non existent slave */
 		if (s != ide_if && !s->bs)
@@ -2308,12 +2290,13 @@ static uint32_t ide_ioport_read(void *opaque, uint32_t addr1)
 	IDEState *ide_if = opaque;
 	IDEState *s = ide_if->cur_drive;
 	uint32_t addr;
-	int ret, hob;
+	int ret;
+
+	/* FIXME: HOB readback uses bit 7, but it's always set right now */
+	//int hob = s->select & (1 << 7);
+	const int hob = 0;
 
 	addr = addr1 & 7;
-	/* FIXME: HOB readback uses bit 7, but it's always set right now */
-	//hob = s->select & (1 << 7);
-	hob = 0;
 	switch (addr)
 	{
 	case 0:
@@ -2372,13 +2355,12 @@ static uint32_t ide_ioport_read(void *opaque, uint32_t addr1)
 			ret = 0;
 		else
 			ret = s->status;
-		/* Lower IRQ */
-		MFP_GPIP |= 0x20;
+
+		/* Clear IRQ (set line to high) */
+		MFP_GPIP_Set_Line_Input ( MFP_GPIP_LINE_FDC_HDC , MFP_GPIP_STATE_HIGH );
 		break;
 	}
-#ifdef DEBUG_IDE
-	printf("IDE: read addr=0x%x val=%02x\n", addr1, ret);
-#endif
+	LOG_TRACE(TRACE_IDE, "IDE: read addr=0x%x val=%02x\n", addr1, ret);
 	return ret;
 }
 
@@ -2393,9 +2375,8 @@ static uint32_t ide_status_read(void *opaque, uint32_t addr)
 		ret = 0;
 	else
 		ret = s->status;
-#ifdef DEBUG_IDE
-	printf("IDE: read status addr=0x%x val=%02x\n", addr, ret);
-#endif
+
+	LOG_TRACE(TRACE_IDE, "IDE: read status addr=0x%x val=%02x\n", addr, ret);
 	return ret;
 }
 
@@ -2405,9 +2386,8 @@ static void ide_cmd_write(void *opaque, uint32_t addr, uint32_t val)
 	IDEState *s;
 	int i;
 
-#ifdef DEBUG_IDE
-	printf("IDE: write control addr=0x%x val=%02x\n", addr, val);
-#endif
+	LOG_TRACE(TRACE_IDE, "IDE: write control addr=0x%x val=%02x\n", addr, val);
+
 	/* common for both drives */
 	if (!(ide_if[0].cmd & IDE_CMD_RESET) &&
 	        (val & IDE_CMD_RESET))
@@ -2575,10 +2555,8 @@ static int guess_disk_lchs(IDEState *s,
 			*pheads = heads;
 			*psectors = sectors;
 			*pcylinders = cylinders;
-#if 0
-			printf("IDE guessed geometry: LCHS=%d %d %d\n",
+			LOG_TRACE(TRACE_IDE, "IDE: guessed geometry LCHS=%d %d %d\n",
 			       cylinders, heads, sectors);
-#endif
 			qemu_free(buf);
 			return 0;
 		}
@@ -2710,10 +2688,12 @@ void Ide_Init(void)
 	memset(hd_table[1], 0, sizeof(BlockDriverState));
 
 	bdrv_open(hd_table[0], ConfigureParams.HardDisk.szIdeMasterHardDiskImage, 0);
+	nIDEPartitions += HDC_PartitionCount(hd_table[0]->fhndl, TRACE_IDE);
 
 	if (ConfigureParams.HardDisk.bUseIdeSlaveHardDiskImage)
 	{
 		bdrv_open(hd_table[1], ConfigureParams.HardDisk.szIdeSlaveHardDiskImage, 0);
+		nIDEPartitions += HDC_PartitionCount(hd_table[1]->fhndl, TRACE_IDE);
 
 		ide_init2(&opaque_ide_if[0], hd_table[0], hd_table[1]);
 	}
@@ -2757,4 +2737,6 @@ void Ide_UnInit(void)
 		free(opaque_ide_if);
 		opaque_ide_if = NULL;
 	}
+
+	nIDEPartitions = 0;
 }
