@@ -8,6 +8,9 @@
 #ifndef HATARI_CONFIGURATION_H
 #define HATARI_CONFIGURATION_H
 
+/* if header's struct contents depend on configuration options, header must include config.h */
+#include "config.h"
+
 /* Logging and tracing */
 typedef struct
 {
@@ -100,6 +103,10 @@ typedef enum {
   SHORTCUT_LOADMEM,
   SHORTCUT_SAVEMEM,
   SHORTCUT_INSERTDISKA,
+  SHORTCUT_JOY_0,
+  SHORTCUT_JOY_1,
+  SHORTCUT_PAD_A,
+  SHORTCUT_PAD_B,
   SHORTCUT_KEYS,  /* number of shortcuts */
   SHORTCUT_NONE
 } SHORTCUTKEYIDX;
@@ -114,6 +121,7 @@ typedef struct
 typedef struct
 {
   int nMemorySize;
+  int nTTRamSize;
   bool bAutoSave;
   char szMemoryCaptureFileName[FILENAME_MAX];
   char szAutoSaveFileName[FILENAME_MAX];
@@ -127,6 +135,7 @@ typedef enum
   JOYSTICK_REALSTICK,
   JOYSTICK_KEYBOARD
 } JOYSTICKMODE;
+#define JOYSTICK_MODES 3
 
 typedef struct
 {
@@ -137,7 +146,16 @@ typedef struct
   int nKeyCodeUp, nKeyCodeDown, nKeyCodeLeft, nKeyCodeRight, nKeyCodeFire;
 } JOYSTICK;
 
-#define JOYSTICK_COUNT 6
+enum
+{
+	JOYID_JOYSTICK0,
+	JOYID_JOYSTICK1,
+	JOYID_STEPADA,
+	JOYID_STEPADB,
+	JOYID_PARPORT1,
+	JOYID_PARPORT2,
+	JOYSTICK_COUNT
+};
 
 typedef struct
 {
@@ -174,6 +192,7 @@ typedef struct
 /* Hard drives configuration: C: - Z: */
 #define MAX_HARDDRIVES  24
 #define DRIVE_C 0
+#define DRIVE_SKIP -1
 
 typedef enum
 {
@@ -184,12 +203,13 @@ typedef enum
 
 typedef struct
 {
-  int nHardDiskDir;
+  int nGemdosDrive;
   bool bUseHardDiskDirectories;
   bool bUseIdeMasterHardDiskImage;
   bool bUseIdeSlaveHardDiskImage;
   WRITEPROTECTION nWriteProtection;
   GEMDOS_CHR_CONV nGemdosCase;
+  bool bFilenameConversion;
   bool bBootFromHardDisk;
   char szHardDiskDirectories[MAX_HARDDRIVES][FILENAME_MAX];
   char szIdeMasterHardDiskImage[FILENAME_MAX];
@@ -198,6 +218,7 @@ typedef struct
 
 /* SCSI/ACSI configuration */
 #define MAX_ACSI_DEVS 8
+#define MAX_SCSI_DEVS 8
 
 typedef struct
 {
@@ -240,12 +261,17 @@ typedef struct
   int nVdiColors;
   int nVdiWidth;
   int nVdiHeight;
+  bool bMouseWarp;
   bool bShowStatusbar;
   bool bShowDriveLed;
   bool bCrop;
   bool bForceMax;
   int nMaxWidth;
   int nMaxHeight;
+#if WITH_SDL2
+  int nRenderScaleQuality;
+  bool bUseVsync;
+#endif
 } CNF_SCREEN;
 
 
@@ -305,9 +331,9 @@ typedef struct
   bool bPatchTimerD;
   bool bFastBoot;                 /* Enable to patch TOS for fast boot */
   bool bFastForward;
+  bool bAddressSpace24;           /* Always set to true with old UAE cpu */
 
 #if ENABLE_WINUAE_CPU
-  bool bAddressSpace24;
   bool bCycleExactCpu;
   FPUTYPE n_FPUType;
   bool bCompatibleFPU;            /* More compatible FPU */
@@ -338,6 +364,7 @@ typedef struct
   CNF_DISKIMAGE DiskImage;
   CNF_HARDDISK HardDisk;
   CNF_SCSIDEV Acsi[MAX_ACSI_DEVS];
+  CNF_SCSIDEV Scsi[MAX_SCSI_DEVS];
   CNF_ROM Rom;
   CNF_RS232 RS232;
   CNF_PRINTER Printer;
