@@ -49,7 +49,7 @@
 
 #define ACSI_EMU_ON        bAcsiEmuOn         /* Do we have HDC emulation? */
 
-extern int nPartitions;
+extern int nAcsiPartitions;
 extern bool bAcsiEmuOn;
 
 /**
@@ -60,6 +60,7 @@ extern void HDC_UnInit(void);
 extern void HDC_ResetCommandStatus(void);
 extern short int HDC_ReadCommandByte(int addr);
 extern void HDC_WriteCommandByte(int addr, Uint8 byte);
+extern int HDC_PartitionCount(FILE *fp, const Uint64 tracelevel);
 
 void Ncr5380_Reset(void);
 
