@@ -27,6 +27,13 @@
 # define unlikely(x)    (x)
 #endif
 
+/* avoid warnings with variables used only in asserts */
+#ifdef NDEBUG
+# define ASSERT_VARIABLE(x) (void)(x)
+#else
+# define ASSERT_VARIABLE(x) assert(x)
+#endif
+
 #ifdef WIN32
 #define PATHSEP '\\'
 #else
@@ -55,7 +62,7 @@ extern void Main_RequestQuit(int exitval);
 extern void Main_SetRunVBLs(Uint32 vbls);
 extern bool Main_SetVBLSlowdown(int factor);
 extern void Main_WaitOnVbl(void);
-extern void Main_WarpMouse(int x, int y);
+extern void Main_WarpMouse(int x, int y, bool restore);
 extern void Main_EventHandler(void);
 extern void Main_SetTitle(const char *title);
 
