@@ -21,6 +21,13 @@ const char Joy_fileid[] = "Hatari joy.c : " __DATE__ " " __TIME__;
 #include "video.h"
 #include "statusbar.h"
 
+#define JOY_DEBUG 0
+#if JOY_DEBUG
+#define Dprintf(a) printf a
+#else
+#define Dprintf(a)
+#endif
+
 #define JOY_BUTTON1  1
 #define JOY_BUTTON2  2
 
@@ -58,6 +65,46 @@ static Uint8 nJoyKeyEmu[ JOYSTICK_COUNT ];
 static Uint16 nSteJoySelect;
 
 
+/**
+ * Get joystick name
+ */
+const char *Joy_GetName(int id)
+{
+#if WITH_SDL2
+	return SDL_JoystickName(sdlJoystick[id]);
+#else
+	return SDL_JoystickName(id);
+#endif
+}
+
+/**
+ * Return maximum available real joystick ID
+ */
+int Joy_GetMaxId(void)
+{
+	int count = SDL_NumJoysticks();
+	if (count > JOYSTICK_COUNT)
+		count = JOYSTICK_COUNT;
+	return count - 1;
+}
+
+/**
+ * Make sure real Joystick ID is valid, and if not, disable it & return false
+ */
+bool Joy_ValidateJoyId(int i)
+{
+	/* Unavailable joystick ID -> disable it if necessary */
+	if (ConfigureParams.Joysticks.Joy[i].nJoystickMode == JOYSTICK_REALSTICK &&
+	    !bJoystickWorking[ConfigureParams.Joysticks.Joy[i].nJoyId])
+	{
+		Log_Printf(LOG_WARN, "Selected real Joystick %d unavailable, disabling ST joystick %d\n", ConfigureParams.Joysticks.Joy[i].nJoyId, i);
+		ConfigureParams.Joysticks.Joy[i].nJoystickMode = JOYSTICK_DISABLED;
+		ConfigureParams.Joysticks.Joy[i].nJoyId = 0;
+		return false;
+	}
+	return true;
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * This function initialises the (real) joysticks.
@@ -104,11 +151,11 @@ void Joy_Init(void)
 		{
 			/* Set as working */
 			bJoystickWorking[i] = true;
-			Log_Printf(LOG_DEBUG, "Joystick %i: %s\n", i, SDL_JoystickName(i));
+			Log_Printf(LOG_DEBUG, "Joystick %i: %s\n", i, Joy_GetName(i));
 			/* determine joystick axis mapping for given SDL joystick name, last is default: */
 			for (j = 0; j < ARRAYSIZE(AxisMappingTable)-1; j++) {
 				/* check if ID string matches the one reported by SDL: */
-				if(strncmp(AxisMappingTable[j].SDLJoystickName, SDL_JoystickName(i), strlen(AxisMappingTable[j].SDLJoystickName)) == 0)
+				if(strncmp(AxisMappingTable[j].SDLJoystickName, Joy_GetName(i), strlen(AxisMappingTable[j].SDLJoystickName)) == 0)
 					break;
 			}
 
@@ -118,6 +165,9 @@ void Joy_Init(void)
 		}
 	}
 
+	for (i = 0; i < JOYSTICK_COUNT ; i++)
+		Joy_ValidateJoyId(i);
+
 	JoystickSpaceBar = false;
 }
 
@@ -310,10 +360,10 @@ static int Joy_GetFireButtons(int nStJoyId)
 bool Joy_SetCursorEmulation(int port)
 {
 	if (port < 0 || port >= JOYSTICK_COUNT) {
-		return 0;
+		return false;
 	}
 	ConfigureParams.Joysticks.Joy[port].nJoystickMode = JOYSTICK_KEYBOARD;
-	return 1;
+	return true;
 }
 
 
@@ -351,6 +401,23 @@ void Joy_ToggleCursorEmulation(void)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Switch between joystick types in given joyport
+ */
+bool Joy_SwitchMode(int port)
+{
+	int mode;
+	if (port < 0 || port >= JOYSTICK_COUNT) {
+		return false;
+	}
+	mode = (ConfigureParams.Joysticks.Joy[port].nJoystickMode + 1) % JOYSTICK_MODES;
+	ConfigureParams.Joysticks.Joy[port].nJoystickMode = mode;
+	Statusbar_UpdateInfo();
+	return true;
+}
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * A key has been pressed down, check if we use it for joystick emulation
@@ -478,7 +545,7 @@ void Joy_StePadButtons_ReadWord(void)
 		}
 		else if (!(nSteJoySelect & 0x8))
 		{
-			if (nButtons & 0x01)  /* Fire button OPTION pressed? */
+			if (nButtons & 0x08)  /* Fire button OPTION pressed? */
 				nData &= ~2;
 		}
 	}
@@ -511,6 +578,7 @@ void Joy_StePadButtons_ReadWord(void)
 		}
 	}
 
+	Dprintf(("0xff9200 -> 0x%04x\n", nData));
 	IoMem_WriteWord(0xff9200, nData);
 }
 
@@ -518,10 +586,12 @@ void Joy_StePadButtons_ReadWord(void)
 /*-----------------------------------------------------------------------*/
 /**
  * Read from STE joypad direction/buttons register (0xff9202)
+ *
+ * This is used e.g. by Reservoir Gods' Tautology II
  */
 void Joy_StePadMulti_ReadWord(void)
 {
-	Uint8 nData = 0xff;
+	Uint16 nData = 0xff;
 
 	if (ConfigureParams.Joysticks.Joy[JOYID_STEPADA].nJoystickMode != JOYSTICK_DISABLED
 	    && (nSteJoySelect & 0x0f) != 0x0f)
@@ -555,19 +625,21 @@ void Joy_StePadMulti_ReadWord(void)
 		}
 		else if (!(nSteJoySelect & 0x20))
 		{
-			nData |= ~(Joy_GetFireButtons(JOYID_STEPADB) >> 13) & 0x0f;
+			nData |= ~(Joy_GetFireButtons(JOYID_STEPADB) >> (13-4)) & 0xf0;
 		}
 		else if (!(nSteJoySelect & 0x40))
 		{
-			nData |= ~(Joy_GetFireButtons(JOYID_STEPADB) >> 9) & 0x0f;
+			nData |= ~(Joy_GetFireButtons(JOYID_STEPADB) >> (9-4)) & 0xf0;
 		}
 		else if (!(nSteJoySelect & 0x80))
 		{
-			nData |= ~(Joy_GetFireButtons(JOYID_STEPADB) >> 5) & 0x0f;
+			nData |= ~(Joy_GetFireButtons(JOYID_STEPADB) >> (5-4)) & 0xf0;
 		}
 	}
 
-	IoMem_WriteWord(0xff9202, (nData << 8) | 0x0ff);
+	nData = (nData << 8) | 0x0ff;
+	Dprintf(("0xff9202 -> 0x%04x\n", nData));
+	IoMem_WriteWord(0xff9202, nData);
 }
 
 
@@ -578,4 +650,5 @@ void Joy_StePadMulti_ReadWord(void)
 void Joy_StePadMulti_WriteWord(void)
 {
 	nSteJoySelect = IoMem_ReadWord(0xff9202);
+	Dprintf(("0xff9202 <- 0x%04x\n", nSteJoySelect));
 }
