@@ -22,302 +22,13 @@ const char Keymap_fileid[] = "Hatari keymap.c : " __DATE__ " " __TIME__;
 #include "log.h"
 
 
-/*-----------------------------------------------------------------------*/
-/*
-  Remap table of PC keys to ST scan codes, -ve is invalid key (ie doesn't occur on ST)
-
-  PC Keyboard:-
-
-    Esc  F1  F2  F3  F4    F5  F6  F7  F8    F9  F10 F11 F12       Print Scroll Pause
-
-     1   59  60  61  62    63  64  65  66    67  68  87  88                70     69
-
-
-   !   "       $   %   ^   &   *   (   )   _   +                                 Page
-`  1   2   3   4   5   6   7   8   9   0   -   =   <-               Ins   Home    Up
-
-41 2   3   4   5   6   7   8   9   10  11  12  13  14               82     71     73
-                                                                    --     --     --
-                                                       |
-                                             {   }     |                         Page
-Tab  Q   W   E   R   T   Y   U   I   O   P   [   ] <----            Del   End    Down
-
-15   16  17  18  19  20  21  22  23  24  25  26  27  28             83     79     81
-                                                                    --     --     --
-
-                                           :   @   ~                       ^
-Caps   A   S   D   F   G   H   J   K   L   ;   '   #                       |
-
-58     30  31  32  33  34  35  36  37  38  39  40  43                      72
-                                                                           --
-
-^   |                               <   >   ?   ^
-|   \   Z   X   C   V   B   N   M   ,   .   /   |                     <-   |   ->
-
-42  86  44  45  46  47  48  49  50  51  52  53  54                    75   80  77
-                                                                      --   --  --
-
-Ctrl  Alt          SPACE             Alt Gr      Ctrl
-
- 29    56            57                56         29
-                                                  --
-
-  And:-
-
-Num
-Lock   /    *    -
-
-69     53   55   74
---     --
-
- 7    8     9    +
-Home  ^   Pg Up
-
-71    72    73   78
-
-
-4     5     6
-<-          ->
-
-75    76    77
-
-
-1     2     3
-End   |   Pg Dn  Enter
-
-79    70    81    28
-                  --
-
-0     .
-Ins   Del
-
-82    83
-
-
-*/
-
-
-/* SDL symbolic key to ST scan code mapping table */
-static const char SymbolicKeyToSTScanCode[SDLK_LAST] =
-{
-	/* ST,  PC Code */
-	-1,    /* 0 */
-	-1,    /* 1 */
-	-1,    /* 2 */
-	-1,    /* 3 */
-	-1,    /* 4 */
-	-1,    /* 5 */
-	-1,    /* 6 */
-	-1,    /* 7 */
-	0x0E,  /* SDLK_BACKSPACE=8 */
-	0x0F,  /* SDLK_TAB=9 */
-	-1,    /* 10 */
-	-1,    /* 11 */
-	0x47,  /* SDLK_CLEAR = 12 */
-	0x1C,  /* SDLK_RETURN = 13 */
-	-1,    /* 14 */
-	-1,    /* 15 */
-	-1,    /* 16 */
-	-1,    /* 17 */
-	-1,    /* 18 */
-	-1,    /* SDLK_PAUSE = 19 */
-	-1,    /* 20 */
-	-1,    /* 21 */
-	-1,    /* 22 */
-	-1,    /* 23 */
-	-1,    /* 24 */
-	-1,    /* 25 */
-	-1,    /* 26 */
-	0x01,  /* SDLK_ESCAPE = 27 */
-	-1,    /* 28 */
-	-1,    /* 29 */
-	-1,    /* 30 */
-	-1,    /* 31 */
-	0x39,  /* SDLK_SPACE = 32 */
-	-1,    /* SDLK_EXCLAIM = 33 */
-	-1,    /* SDLK_QUOTEDBL = 34 */
-	0x29,  /* SDLK_HASH = 35 */
-	-1,    /* SDLK_DOLLAR = 36 */
-	-1,    /* 37 */
-	-1,    /* SDLK_AMPERSAND = 38 */
-	0x28,  /* SDLK_QUOTE = 39 */
-	0x63,  /* SDLK_LEFTPAREN = 40 */
-	0x64,  /* SDLK_RIGHTPAREN = 41 */
-	-1,    /* SDLK_ASTERISK = 42 */
-	0x1B,  /* SDLK_PLUS = 43 */
-	0x33,  /* SDLK_COMMA = 44 */
-	0x35,  /* SDLK_MINUS = 45 */
-	0x34,  /* SDLK_PERIOD = 46 */
-	0x35,  /* SDLK_SLASH = 47 */
-	0x0B,  /* SDLK_0 = 48 */
-	0x02,  /* SDLK_1 = 49 */
-	0x03,  /* SDLK_2 = 50 */
-	0x04,  /* SDLK_3 = 51 */
-	0x05,  /* SDLK_4 = 52 */
-	0x06,  /* SDLK_5 = 53 */
-	0x07,  /* SDLK_6 = 54 */
-	0x08,  /* SDLK_7 = 55 */
-	0x09,  /* SDLK_8 = 56 */
-	0x0A,  /* SDLK_9 = 57 */
-	-1,    /* SDLK_COLON = 58 */
-	0x27,  /* SDLK_SEMICOLON = 59 */
-	0x60,  /* SDLK_LESS = 60 */
-	0x0D,  /* SDLK_EQUALS = 61 */
-	-1,    /* SDLK_GREATER  = 62 */
-	-1,    /* SDLK_QUESTION = 63 */
-	-1,    /* SDLK_AT = 64 */
-	-1,    /* 65 */  /* Skip uppercase letters */
-	-1,    /* 66 */
-	-1,    /* 67 */
-	-1,    /* 68 */
-	-1,    /* 69 */
-	-1,    /* 70 */
-	-1,    /* 71 */
-	-1,    /* 72 */
-	-1,    /* 73 */
-	-1,    /* 74 */
-	-1,    /* 75 */
-	-1,    /* 76 */
-	-1,    /* 77 */
-	-1,    /* 78 */
-	-1,    /* 79 */
-	-1,    /* 80 */
-	-1,    /* 81 */
-	-1,    /* 82 */
-	-1,    /* 83 */
-	-1,    /* 84 */
-	-1,    /* 85 */
-	-1,    /* 86 */
-	-1,    /* 87 */
-	-1,    /* 88 */
-	-1,    /* 89 */
-	-1,    /* 90 */
-	0x63,  /* SDLK_LEFTBRACKET = 91 */
-	0x2B,  /* SDLK_BACKSLASH = 92 */     /* Might be 0x60 for UK keyboards */
-	0x64,  /* SDLK_RIGHTBRACKET = 93 */
-	0x2B,  /* SDLK_CARET = 94 */
-	-1,    /* SDLK_UNDERSCORE = 95 */
-	0x52,  /* SDLK_BACKQUOTE = 96 */
-	0x1E,  /* SDLK_a = 97 */
-	0x30,  /* SDLK_b = 98 */
-	0x2E,  /* SDLK_c = 99 */
-	0x20,  /* SDLK_d = 100 */
-	0x12,  /* SDLK_e = 101 */
-	0x21,  /* SDLK_f = 102 */
-	0x22,  /* SDLK_g = 103 */
-	0x23,  /* SDLK_h = 104 */
-	0x17,  /* SDLK_i = 105 */
-	0x24,  /* SDLK_j = 106 */
-	0x25,  /* SDLK_k = 107 */
-	0x26,  /* SDLK_l = 108 */
-	0x32,  /* SDLK_m = 109 */
-	0x31,  /* SDLK_n = 110 */
-	0x18,  /* SDLK_o = 111 */
-	0x19,  /* SDLK_p = 112 */
-	0x10,  /* SDLK_q = 113 */
-	0x13,  /* SDLK_r = 114 */
-	0x1F,  /* SDLK_s = 115 */
-	0x14,  /* SDLK_t = 116 */
-	0x16,  /* SDLK_u = 117 */
-	0x2F,  /* SDLK_v = 118 */
-	0x11,  /* SDLK_w = 119 */
-	0x2D,  /* SDLK_x = 120 */
-	0x15,  /* SDLK_y = 121 */
-	0x2C,  /* SDLK_z = 122 */
-	-1,    /* 123 */
-	-1,    /* 124 */
-	-1,    /* 125 */
-	-1,    /* 126 */
-	0x53,  /* SDLK_DELETE = 127 */
-	/* End of ASCII mapped keysyms */
-	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 128-143*/
-	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 144-159*/
-	0x0d, 0x0c, 0x1a, 0x28, 0x27, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 160-175*/
-	-1, -1, -1, -1, 0x0D, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 176-191*/
-	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 192-207*/
-	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 0x0C, /* 208-223*/
-	-1, -1, -1, -1, 0x28, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /* 224-239*/
-	-1, -1, -1, -1, -1, -1, 0x27, -1, -1, -1, -1, -1, 0x1A, -1, -1, -1, /* 240-255*/
-	/* Numeric keypad: */
-	0x70,    /* SDLK_KP0 = 256 */
-	0x6D,    /* SDLK_KP1 = 257 */
-	0x6E,    /* SDLK_KP2 = 258 */
-	0x6F,    /* SDLK_KP3 = 259 */
-	0x6A,    /* SDLK_KP4 = 260 */
-	0x6B,    /* SDLK_KP5 = 261 */
-	0x6C,    /* SDLK_KP6 = 262 */
-	0x67,    /* SDLK_KP7 = 263 */
-	0x68,    /* SDLK_KP8 = 264 */
-	0x69,    /* SDLK_KP9 = 265 */
-	0x71,    /* SDLK_KP_PERIOD = 266 */
-	0x65,    /* SDLK_KP_DIVIDE = 267 */
-	0x66,    /* SDLK_KP_MULTIPLY = 268 */
-	0x4A,    /* SDLK_KP_MINUS = 269 */
-	0x4E,    /* SDLK_KP_PLUS = 270 */
-	0x72,    /* SDLK_KP_ENTER = 271 */
-	0x61,    /* SDLK_KP_EQUALS = 272 */
-	/* Arrows + Home/End pad */
-	0x48,    /* SDLK_UP = 273 */
-	0x50,    /* SDLK_DOWN = 274 */
-	0x4D,    /* SDLK_RIGHT = 275 */
-	0x4B,    /* SDLK_LEFT = 276 */
-	0x52,    /* SDLK_INSERT = 277 */
-	0x47,    /* SDLK_HOME = 278 */
-	0x61,    /* SDLK_END = 279 */
-	0x63,    /* SDLK_PAGEUP = 280 */
-	0x64,    /* SDLK_PAGEDOWN = 281 */
-	/* Function keys */
-	0x3B,    /* SDLK_F1 = 282 */
-	0x3C,    /* SDLK_F2 = 283 */
-	0x3D,    /* SDLK_F3 = 284 */
-	0x3E,    /* SDLK_F4 = 285 */
-	0x3F,    /* SDLK_F5 = 286 */
-	0x40,    /* SDLK_F6 = 287 */
-	0x41,    /* SDLK_F7 = 288 */
-	0x42,    /* SDLK_F8 = 289 */
-	0x43,    /* SDLK_F9 = 290 */
-	0x44,    /* SDLK_F10 = 291 */
-	-1,      /* SDLK_F11 = 292 */
-	-1,      /* SDLK_F12 = 293 */
-	0x62,    /* SDLK_F13 = 294 */
-	-1,      /* SDLK_F14 = 295 */
-	-1,      /* SDLK_F15 = 296 */
-	-1,      /* 297 */
-	-1,      /* 298 */
-	-1,      /* 299 */
-	/* Key state modifier keys */
-	-1,      /* SDLK_NUMLOCK = 300 */
-	0x3A,    /* SDLK_CAPSLOCK = 301 */
-	0x61,    /* SDLK_SCROLLOCK = 302 */
-	0x36,    /* SDLK_RSHIFT = 303 */
-	0x2A,    /* SDLK_LSHIFT = 304 */
-	0x1D,    /* SDLK_RCTRL = 305 */
-	0x1D,    /* SDLK_LCTRL = 306 */
-	0x38,    /* SDLK_RALT = 307 */
-	0x38,    /* SDLK_LALT = 308 */
-	-1,      /* SDLK_RMETA = 309 */
-	-1,      /* SDLK_LMETA = 310 */
-	-1,      /* SDLK_LSUPER = 311 */
-	-1,      /* SDLK_RSUPER = 312 */
-	-1,      /* SDLK_MODE = 313 */     /* "Alt Gr" key */
-	-1,      /* SDLK_COMPOSE = 314 */
-	/* Miscellaneous function keys */
-	0x62,    /* SDLK_HELP = 315 */
-	0x62,    /* SDLK_PRINT = 316 */
-	-1,      /* SDLK_SYSREQ = 317 */
-	-1,      /* SDLK_BREAK = 318 */
-	-1,      /* SDLK_MENU = 319 */
-	-1,      /* SDLK_POWER = 320 */
-	-1,      /* SDLK_EURO = 321 */
-	0x61     /* SDLK_UNDO = 322 */
-};
-
-/* Table for loaded keys: */
-static char LoadedKeyToSTScanCode[SDLK_LAST];
-
+#if !WITH_SDL2
 /* This table is used to translate a symbolic keycode to the (SDL) scancode */
 static Uint8 SdlSymToSdlScan[SDLK_LAST];
+#endif
 
+/* Table for loaded keys: */
+static int LoadedKeymap[KBD_MAX_SCANCODE][2];
 
 /* List of ST scan codes to NOT de-bounce when running in maximum speed */
 static const char DebounceExtendedKeys[] =
@@ -338,12 +49,305 @@ static const char DebounceExtendedKeys[] =
  */
 void Keymap_Init(void)
 {
+#if !WITH_SDL2
 	memset(SdlSymToSdlScan, 0, sizeof(SdlSymToSdlScan));      /* Clear array */
+#endif
 	Keymap_LoadRemapFile(ConfigureParams.Keyboard.szMappingFileName);
 }
 
+/**
+ * Map SDL symbolic key to ST scan code
+ */
+static const char Keymap_SymbolicToStScanCode(SDL_keysym* pKeySym)
+{
+	char code;
+
+	switch (pKeySym->sym)
+	{
+	 case SDLK_BACKSPACE: code = 0x0E; break;
+	 case SDLK_TAB: code = 0x0F; break;
+	 case SDLK_CLEAR: code = 0x47; break;
+	 case SDLK_RETURN: code = 0x1C; break;
+	 case SDLK_ESCAPE: code = 0x01; break;
+	 case SDLK_SPACE: code = 0x39; break;
+	 case SDLK_EXCLAIM: code = 0x09; break;     /* on azerty? */
+	 case SDLK_QUOTEDBL: code = 0x04; break;    /* on azerty? */
+	 case SDLK_HASH: code = 0x29; break;
+	 case SDLK_DOLLAR: code = 0x1b; break;      /* on azerty */
+	 case SDLK_AMPERSAND: code = 0x02; break;   /* on azerty? */
+	 case SDLK_QUOTE: code = 0x28; break;
+	 case SDLK_LEFTPAREN: code = 0x63; break;
+	 case SDLK_RIGHTPAREN: code = 0x64; break;
+	 case SDLK_ASTERISK: code = 0x66; break;
+	 case SDLK_PLUS: code = 0x1B; break;
+	 case SDLK_COMMA: code = 0x33; break;
+	 case SDLK_MINUS: code = 0x35; break;
+	 case SDLK_PERIOD: code = 0x34; break;
+	 case SDLK_SLASH: code = 0x35; break;
+	 case SDLK_0: code = 0x0B; break;
+	 case SDLK_1: code = 0x02; break;
+	 case SDLK_2: code = 0x03; break;
+	 case SDLK_3: code = 0x04; break;
+	 case SDLK_4: code = 0x05; break;
+	 case SDLK_5: code = 0x06; break;
+	 case SDLK_6: code = 0x07; break;
+	 case SDLK_7: code = 0x08; break;
+	 case SDLK_8: code = 0x09; break;
+	 case SDLK_9: code = 0x0A; break;
+	 case SDLK_COLON: code = 0x34; break;
+	 case SDLK_SEMICOLON: code = 0x27; break;
+	 case SDLK_LESS: code = 0x60; break;
+	 case SDLK_EQUALS: code = 0x0D; break;
+	 //case SDLK_GREATER : code = -1; break;
+	 //case SDLK_QUESTION: code = -1; break;
+	 //case SDLK_AT: code = -1; break;
+	 case SDLK_LEFTBRACKET: code = 0x63; break;
+	 case SDLK_BACKSLASH: code = 0x2B; break;     /* Might be 0x60 for UK keyboards */
+	 case SDLK_RIGHTBRACKET: code = 0x64; break;
+	 case SDLK_CARET: code = 0x2B; break;
+	 //case SDLK_UNDERSCORE: code = -1; break;
+	 case SDLK_BACKQUOTE: code = 0x52; break;
+	 case SDLK_a: code = 0x1E; break;
+	 case SDLK_b: code = 0x30; break;
+	 case SDLK_c: code = 0x2E; break;
+	 case SDLK_d: code = 0x20; break;
+	 case SDLK_e: code = 0x12; break;
+	 case SDLK_f: code = 0x21; break;
+	 case SDLK_g: code = 0x22; break;
+	 case SDLK_h: code = 0x23; break;
+	 case SDLK_i: code = 0x17; break;
+	 case SDLK_j: code = 0x24; break;
+	 case SDLK_k: code = 0x25; break;
+	 case SDLK_l: code = 0x26; break;
+	 case SDLK_m: code = 0x32; break;
+	 case SDLK_n: code = 0x31; break;
+	 case SDLK_o: code = 0x18; break;
+	 case SDLK_p: code = 0x19; break;
+	 case SDLK_q: code = 0x10; break;
+	 case SDLK_r: code = 0x13; break;
+	 case SDLK_s: code = 0x1F; break;
+	 case SDLK_t: code = 0x14; break;
+	 case SDLK_u: code = 0x16; break;
+	 case SDLK_v: code = 0x2F; break;
+	 case SDLK_w: code = 0x11; break;
+	 case SDLK_x: code = 0x2D; break;
+	 case SDLK_y: code = 0x15; break;
+	 case SDLK_z: code = 0x2C; break;
+	 case SDLK_DELETE: code = 0x53; break;
+	 /* End of ASCII mapped keysyms */
+#if WITH_SDL2
+	 case 223: code = 0x0C; break;
+	 case 228: code = 0x28; break;
+	 case 246: code = 0x27; break;
+	 case 252: code = 0x1A; break;
+#else
+	 case SDLK_WORLD_0: code = 0x0d; break;
+	 case SDLK_WORLD_1: code = 0x0c; break;
+	 case SDLK_WORLD_2: code = 0x1a; break;
+	 case SDLK_WORLD_3: code = 0x28; break;
+	 case SDLK_WORLD_4: code = 0x27; break;
+	 case SDLK_WORLD_20: code = 0x0D; break;
+	 case SDLK_WORLD_63: code = 0x0C; break;
+	 case SDLK_WORLD_68: code = 0x28; break;
+	 case SDLK_WORLD_86: code = 0x27; break;
+	 case SDLK_WORLD_92: code = 0x1A; break;
+#endif
+	 /* Numeric keypad: */
+	 case SDLK_KP0: code = 0x70; break;
+	 case SDLK_KP1: code = 0x6D; break;
+	 case SDLK_KP2: code = 0x6E; break;
+	 case SDLK_KP3: code = 0x6F; break;
+	 case SDLK_KP4: code = 0x6A; break;
+	 case SDLK_KP5: code = 0x6B; break;
+	 case SDLK_KP6: code = 0x6C; break;
+	 case SDLK_KP7: code = 0x67; break;
+	 case SDLK_KP8: code = 0x68; break;
+	 case SDLK_KP9: code = 0x69; break;
+	 case SDLK_KP_PERIOD: code = 0x71; break;
+	 case SDLK_KP_DIVIDE: code = 0x65; break;
+	 case SDLK_KP_MULTIPLY: code = 0x66; break;
+	 case SDLK_KP_MINUS: code = 0x4A; break;
+	 case SDLK_KP_PLUS: code = 0x4E; break;
+	 case SDLK_KP_ENTER: code = 0x72; break;
+	 case SDLK_KP_EQUALS: code = 0x61; break;
+	 /* Arrows + Home/End pad */
+	 case SDLK_UP: code = 0x48; break;
+	 case SDLK_DOWN: code = 0x50; break;
+	 case SDLK_RIGHT: code = 0x4D; break;
+	 case SDLK_LEFT: code = 0x4B; break;
+	 case SDLK_INSERT: code = 0x52; break;
+	 case SDLK_HOME: code = 0x47; break;
+	 case SDLK_END: code = 0x61; break;
+	 case SDLK_PAGEUP: code = 0x63; break;
+	 case SDLK_PAGEDOWN: code = 0x64; break;
+	 /* Function keys */
+	 case SDLK_F1: code = 0x3B; break;
+	 case SDLK_F2: code = 0x3C; break;
+	 case SDLK_F3: code = 0x3D; break;
+	 case SDLK_F4: code = 0x3E; break;
+	 case SDLK_F5: code = 0x3F; break;
+	 case SDLK_F6: code = 0x40; break;
+	 case SDLK_F7: code = 0x41; break;
+	 case SDLK_F8: code = 0x42; break;
+	 case SDLK_F9: code = 0x43; break;
+	 case SDLK_F10: code = 0x44; break;
+	 case SDLK_F13: code = 0x62; break;
+	 /* Key state modifier keys */
+	 case SDLK_CAPSLOCK: code = 0x3A; break;
+	 case SDLK_SCROLLOCK: code = 0x61; break;
+	 case SDLK_RSHIFT: code = 0x36; break;
+	 case SDLK_LSHIFT: code = 0x2A; break;
+	 case SDLK_RCTRL: code = 0x1D; break;
+	 case SDLK_LCTRL: code = 0x1D; break;
+	 case SDLK_RALT: code = 0x38; break;
+	 case SDLK_LALT: code = 0x38; break;
+	 /* Miscellaneous function keys */
+	 case SDLK_HELP: code = 0x62; break;
+	 case SDLK_PRINT: code = 0x62; break;
+	 case SDLK_UNDO: code = 0x61; break;
+	 default: code = -1;
+	}
+
+	return code;
+}
+
+
+#if WITH_SDL2
+
+/**
+ * Remap SDL scancode key to ST Scan code - this is the version for SDL2
+ */
+static char Keymap_PcToStScanCode(SDL_keysym* pKeySym)
+{
+	switch (pKeySym->scancode)
+	{
+	 case SDL_SCANCODE_A: return 0x1e;
+	 case SDL_SCANCODE_B: return 0x30;
+	 case SDL_SCANCODE_C: return 0x2e;
+	 case SDL_SCANCODE_D: return 0x20;
+	 case SDL_SCANCODE_E: return 0x12;
+	 case SDL_SCANCODE_F: return 0x21;
+	 case SDL_SCANCODE_G: return 0x22;
+	 case SDL_SCANCODE_H: return 0x23;
+	 case SDL_SCANCODE_I: return 0x17;
+	 case SDL_SCANCODE_J: return 0x24;
+	 case SDL_SCANCODE_K: return 0x25;
+	 case SDL_SCANCODE_L: return 0x26;
+	 case SDL_SCANCODE_M: return 0x32;
+	 case SDL_SCANCODE_N: return 0x31;
+	 case SDL_SCANCODE_O: return 0x18;
+	 case SDL_SCANCODE_P: return 0x19;
+	 case SDL_SCANCODE_Q: return 0x10;
+	 case SDL_SCANCODE_R: return 0x13;
+	 case SDL_SCANCODE_S: return 0x1f;
+	 case SDL_SCANCODE_T: return 0x14;
+	 case SDL_SCANCODE_U: return 0x16;
+	 case SDL_SCANCODE_V: return 0x2f;
+	 case SDL_SCANCODE_W: return 0x11;
+	 case SDL_SCANCODE_X: return 0x2d;
+	 case SDL_SCANCODE_Y: return 0x15;
+	 case SDL_SCANCODE_Z: return 0x2c;
+	 case SDL_SCANCODE_1: return 0x02;
+	 case SDL_SCANCODE_2: return 0x03;
+	 case SDL_SCANCODE_3: return 0x04;
+	 case SDL_SCANCODE_4: return 0x05;
+	 case SDL_SCANCODE_5: return 0x06;
+	 case SDL_SCANCODE_6: return 0x07;
+	 case SDL_SCANCODE_7: return 0x08;
+	 case SDL_SCANCODE_8: return 0x09;
+	 case SDL_SCANCODE_9: return 0x0a;
+	 case SDL_SCANCODE_0: return 0x0b;
+	 case SDL_SCANCODE_RETURN: return 0x1c;
+	 case SDL_SCANCODE_ESCAPE: return 0x01;
+	 case SDL_SCANCODE_BACKSPACE: return 0x0e;
+	 case SDL_SCANCODE_TAB: return 0x0f;
+	 case SDL_SCANCODE_SPACE: return 0x39;
+	 case SDL_SCANCODE_MINUS: return 0x0c;
+	 case SDL_SCANCODE_EQUALS: return 0x0d;
+	 case SDL_SCANCODE_LEFTBRACKET: return 0x1a;
+	 case SDL_SCANCODE_RIGHTBRACKET: return 0x1b;
+	 case SDL_SCANCODE_BACKSLASH: return 0x60;
+	 case SDL_SCANCODE_NONUSHASH: return 0x2b;
+	 case SDL_SCANCODE_SEMICOLON: return 0x27;
+	 case SDL_SCANCODE_APOSTROPHE: return 0x28;
+	 case SDL_SCANCODE_GRAVE: return 0x2b;      /* ok? */
+	 case SDL_SCANCODE_COMMA: return 0x33;
+	 case SDL_SCANCODE_PERIOD: return 0x34;
+	 case SDL_SCANCODE_SLASH: return 0x35;
+	 case SDL_SCANCODE_CAPSLOCK: return 0x3a;
+	 case SDL_SCANCODE_F1: return 0x3b;
+	 case SDL_SCANCODE_F2: return 0x3c;
+	 case SDL_SCANCODE_F3: return 0x3d;
+	 case SDL_SCANCODE_F4: return 0x3e;
+	 case SDL_SCANCODE_F5: return 0x3f;
+	 case SDL_SCANCODE_F6: return 0x40;
+	 case SDL_SCANCODE_F7: return 0x41;
+	 case SDL_SCANCODE_F8: return 0x42;
+	 case SDL_SCANCODE_F9: return 0x43;
+	 case SDL_SCANCODE_F10: return 0x44;
+	 case SDL_SCANCODE_PRINTSCREEN: return 0x62;
+	 case SDL_SCANCODE_SCROLLLOCK: return 0x61;
+	 case SDL_SCANCODE_PAUSE: return 0x61;
+	 case SDL_SCANCODE_INSERT: return 0x52;
+	 case SDL_SCANCODE_HOME: return 0x47;
+	 case SDL_SCANCODE_PAGEUP: return 0x62;
+	 case SDL_SCANCODE_DELETE: return 0x53;
+	 case SDL_SCANCODE_END: return 0x2b;
+	 case SDL_SCANCODE_PAGEDOWN: return 0x61;
+	 case SDL_SCANCODE_RIGHT: return 0x4d;
+	 case SDL_SCANCODE_LEFT: return 0x4b;
+	 case SDL_SCANCODE_DOWN: return 0x50;
+	 case SDL_SCANCODE_UP: return 0x48;
+	 case SDL_SCANCODE_NUMLOCKCLEAR: return 0x64;
+	 case SDL_SCANCODE_KP_DIVIDE: return 0x65;
+	 case SDL_SCANCODE_KP_MULTIPLY: return 0x66;
+	 case SDL_SCANCODE_KP_MINUS: return 0x4a;
+	 case SDL_SCANCODE_KP_PLUS: return 0x4e;
+	 case SDL_SCANCODE_KP_ENTER: return 0x72;
+	 case SDL_SCANCODE_KP_1: return 0x6d;
+	 case SDL_SCANCODE_KP_2: return 0x6e;
+	 case SDL_SCANCODE_KP_3: return 0x6f;
+	 case SDL_SCANCODE_KP_4: return 0x6a;
+	 case SDL_SCANCODE_KP_5: return 0x6b;
+	 case SDL_SCANCODE_KP_6: return 0x6c;
+	 case SDL_SCANCODE_KP_7: return 0x67;
+	 case SDL_SCANCODE_KP_8: return 0x68;
+	 case SDL_SCANCODE_KP_9: return 0x69;
+	 case SDL_SCANCODE_KP_0: return 0x70;
+	 case SDL_SCANCODE_KP_PERIOD: return 0x71;
+	 case SDL_SCANCODE_NONUSBACKSLASH: return 0x60;
+	 //case SDL_SCANCODE_APPLICATION: return ;
+	 case SDL_SCANCODE_KP_EQUALS: return 0x63;
+	 case SDL_SCANCODE_F13: return 0x63;
+	 case SDL_SCANCODE_F14: return 0x64;
+	 case SDL_SCANCODE_HELP: return 0x62;
+	 case SDL_SCANCODE_UNDO: return 0x61;
+	 case SDL_SCANCODE_KP_COMMA: return 0x71;
+	 case SDL_SCANCODE_CLEAR: return 0x47;
+	 case SDL_SCANCODE_RETURN2: return 0x1c;
+	 case SDL_SCANCODE_KP_LEFTPAREN: return 0x63;
+	 case SDL_SCANCODE_KP_RIGHTPAREN: return 0x64;
+	 case SDL_SCANCODE_KP_LEFTBRACE: return 0x63;
+	 case SDL_SCANCODE_KP_RIGHTBRACE: return 0x64;
+	 case SDL_SCANCODE_KP_TAB: return 0x0f;
+	 case SDL_SCANCODE_KP_BACKSPACE: return 0x0e;
+	 case SDL_SCANCODE_KP_COLON: return 0x33;
+	 case SDL_SCANCODE_KP_HASH: return 0x0c;
+	 case SDL_SCANCODE_KP_SPACE: return 0x39;
+	 case SDL_SCANCODE_KP_CLEAR: return 0x47;
+	 case SDL_SCANCODE_LCTRL: return 0x1d;
+	 case SDL_SCANCODE_LSHIFT: return 0x2a;
+	 case SDL_SCANCODE_LALT: return 0x38;
+	 case SDL_SCANCODE_RCTRL: return 0x1d;
+	 case SDL_SCANCODE_RSHIFT: return 0x36;
+	 default:
+		fprintf(stderr, "Warning: Unhandled scancode 0x%x!\n", pKeySym->scancode);
+		return -1;
+	}
+}
+
+#else
 
-/*-----------------------------------------------------------------------*/
 /**
  * Heuristic analysis to find out the obscure scancode offset.
  * Some keys like 'z' can't be used for detection since they are on different
@@ -427,7 +431,6 @@ static int Keymap_FindScanCodeOffset(SDL_keysym* keysym)
 }
 
 
-/*-----------------------------------------------------------------------*/
 /**
  * Map PC scancode to ST scancode.
  * This code was heavily inspired by the emulator Aranym. (cheers!)
@@ -436,6 +439,20 @@ static char Keymap_PcToStScanCode(SDL_keysym* keysym)
 {
 	static int offset = -1;    /* uninitialized scancode offset */
 
+	/* We sometimes enter here with an illegal (=0) scancode, so we keep
+	 * track of the right scancodes in a table and then use a value from there.
+	 */
+	if (keysym->scancode != 0)
+	{
+		SdlSymToSdlScan[keysym->sym] = keysym->scancode;
+	}
+	else
+	{
+		keysym->scancode = SdlSymToSdlScan[keysym->sym];
+		if (keysym->scancode == 0)
+			fprintf(stderr, "Warning: Key scancode is 0!\n");
+	}
+
 	switch (keysym->sym)
 	{
 	 /* Numeric Pad */
@@ -493,8 +510,9 @@ static char Keymap_PcToStScanCode(SDL_keysym* keysym)
 	}
 }
 
+#endif	/* WITH_SDL2 */
+
 
-/*-----------------------------------------------------------------------*/
 /**
  * Remap a keypad key to ST scan code. We use a separate function for this
  * so that we can easily toggle between number and cursor mode with the
@@ -541,14 +559,11 @@ static char Keymap_GetKeyPadScanCode(SDL_keysym* pKeySym)
 }
 
 
-/*-----------------------------------------------------------------------*/
 /**
  * Remap SDL Key to ST Scan code
  */
 static char Keymap_RemapKeyToSTScanCode(SDL_keysym* pKeySym)
 {
-	if (pKeySym->sym >= SDLK_LAST)  return -1; /* Avoid illegal keys */
-
 	/* Check for keypad first so we can handle numlock */
 	if (ConfigureParams.Keyboard.nKeymapType != KEYMAP_LOADED)
 	{
@@ -561,28 +576,22 @@ static char Keymap_RemapKeyToSTScanCode(SDL_keysym* pKeySym)
 	/* Remap from PC scancodes? */
 	if (ConfigureParams.Keyboard.nKeymapType == KEYMAP_SCANCODE)
 	{
-		/* We sometimes enter here with an illegal (=0) scancode, so we keep
-		 * track of the right scancodes in a table and then use a value from there.
-		 */
-		if (pKeySym->scancode != 0)
-		{
-			SdlSymToSdlScan[pKeySym->sym] = pKeySym->scancode;
-		}
-		else
-		{
-			pKeySym->scancode = SdlSymToSdlScan[pKeySym->sym];
-			if (pKeySym->scancode == 0)
-				fprintf(stderr, "Warning: Key scancode is 0!\n");
-		}
-
 		return Keymap_PcToStScanCode(pKeySym);
 	}
 
-	/* Use default or loaded? */
+	/* Use loaded keymap? */
 	if (ConfigureParams.Keyboard.nKeymapType == KEYMAP_LOADED)
-		return LoadedKeyToSTScanCode[pKeySym->sym];
-	else
-		return SymbolicKeyToSTScanCode[pKeySym->sym];
+	{
+		int i;
+		for (i = 0; i < KBD_MAX_SCANCODE && LoadedKeymap[i][1] != 0; i++)
+		{
+			if (pKeySym->sym == (SDLKey)LoadedKeymap[i][0])
+				return LoadedKeymap[i][1];
+		}
+	}
+
+	/* Use symbolic mapping */
+	return Keymap_SymbolicToStScanCode(pKeySym);
 }
 
 
@@ -595,9 +604,10 @@ void Keymap_LoadRemapFile(char *pszFileName)
 	char szString[1024];
 	int STScanCode, PCKeyCode;
 	FILE *in;
+	int idx = 0;
 
 	/* Initialize table with default values */
-	memcpy(LoadedKeyToSTScanCode, SymbolicKeyToSTScanCode, sizeof(LoadedKeyToSTScanCode));
+	memset(LoadedKeymap, 0, sizeof(LoadedKeymap));
 
 	if (!*pszFileName)
 		return;
@@ -616,7 +626,7 @@ void Keymap_LoadRemapFile(char *pszFileName)
 		return;
 	}
 	
-	while (!feof(in))
+	while (!feof(in) && idx < KBD_MAX_SCANCODE)
 	{
 		/* Read line from file */
 		if (fgets(szString, sizeof(szString), in) == NULL)
@@ -631,12 +641,16 @@ void Keymap_LoadRemapFile(char *pszFileName)
 			/* Read values */
 			sscanf(szString, "%d,%d", &PCKeyCode, &STScanCode);
 			/* Store into remap table, check both value within range */
-			if ( (PCKeyCode>=0) && (PCKeyCode<SDLK_LAST) &&
-			     (STScanCode>=0) && (STScanCode<256) )
-				LoadedKeyToSTScanCode[PCKeyCode] = STScanCode;
+			if (STScanCode >= 0 && STScanCode <= KBD_MAX_SCANCODE
+			    && PCKeyCode >= 8)
+			{
+				LoadedKeymap[idx][0] = PCKeyCode;
+				LoadedKeymap[idx][1] = STScanCode;
+				idx += 1;
+			}
 		}
 	}
-	
+
 	fclose(in);
 }
 
@@ -717,7 +731,8 @@ void Keymap_KeyDown(SDL_keysym *sdlkey)
 	int symkey = sdlkey->sym;
 	int modkey = sdlkey->mod;
 
-	/*fprintf(stderr, "keydown: sym=%i scan=%i mod=$%x\n",symkey, sdlkey->scancode, modkey);*/
+	LOG_TRACE(TRACE_KEYMAP, "key down: sym=%i scan=%i mod=0x%x\n", symkey,
+	          sdlkey->scancode, modkey);
 
 	if (ShortCut_CheckKeys(modkey, symkey, 1))
 		return;
@@ -736,6 +751,7 @@ void Keymap_KeyDown(SDL_keysym *sdlkey)
 	}
 
 	STScanCode = Keymap_RemapKeyToSTScanCode(sdlkey);
+	LOG_TRACE(TRACE_KEYMAP, "key map: sym=0x%x to ST-scan=0x%02x\n", symkey, STScanCode);
 	if (STScanCode != (uint8_t)-1)
 	{
 		if (!Keyboard.KeyStates[STScanCode])
@@ -758,10 +774,10 @@ void Keymap_KeyUp(SDL_keysym *sdlkey)
 	int symkey = sdlkey->sym;
 	int modkey = sdlkey->mod;
 
-	/*fprintf(stderr, "keyup: sym=%i scan=%i mod=$%x\n",symkey, sdlkey->scancode, modkey);*/
+	LOG_TRACE(TRACE_KEYMAP, "key up: sym=%i scan=%i mod=$%x\n", symkey,
+	          sdlkey->scancode, modkey);
 
 	/* Ignore short-cut keys here */
-
 	if (ShortCut_CheckKeys(modkey, symkey, 0))
 		return;
 
