@@ -61,6 +61,13 @@
 /* 2014/05/07	[NP]	In M68000_WaitEClock, use CyclesGlobalClockCounter instead of the VBL video	*/
 /*			counter (else for a given position in a VBL we would always get the same value	*/
 /*			for the E clock).								*/
+/* 2015/02/01	[NP]	When using the new WinUAE's cpu, don't handle MFP/DSP interrupts by calling	*/
+/*			directly Exception(), we must set bit 6 in pendingInterrupts and use the IACK	*/
+/*			sequence to get the exception's vector number.					*/
+/* 2015/02/05	[NP]	For the new WinUAE's cpu, don't use ExceptionSource anymore when calling	*/
+/*			Exception().									*/
+/* 2015/02/11	[NP]	Replace BusErrorPC by regs.instruction_pc, to get similar code to WinUAE's cpu  */
+
 
 const char M68000_fileid[] = "Hatari m68000.c : " __DATE__ " " __TIME__;
 
@@ -77,6 +84,10 @@ const char M68000_fileid[] = "Hatari m68000.c : " __DATE__ " " __TIME__;
 #include "stMemory.h"
 #include "tos.h"
 
+#if ENABLE_DSP_EMU
+#include "dsp.h"
+#endif
+
 #if ENABLE_WINUAE_CPU
 #include "mmu_common.h"
 #endif
@@ -85,7 +96,6 @@ const char M68000_fileid[] = "Hatari m68000.c : " __DATE__ " " __TIME__;
 cpu_instruction_t CpuInstruction;
 
 Uint32 BusErrorAddress;         /* Stores the offending address for bus-/address errors */
-Uint32 BusErrorPC;              /* Value of the PC when bus error occurs */
 bool bBusErrorReadWrite;        /* 0 for write error, 1 for read error */
 int nCpuFreqShift;              /* Used to emulate higher CPU frequencies: 0=8MHz, 1=16MHz, 2=32Mhz */
 int nWaitStateCycles;           /* Used to emulate the wait state cycles of certain IO registers */
@@ -197,6 +207,9 @@ static void M68000_InitPairing(void)
 
 	PairingArray[ i_ADD ][ i_MOVE ] = 1;		/* when using xx(an,dn) addr mode */
 	PairingArray[ i_SUB ][ i_MOVE ] = 1;
+
+	PairingArray[ i_ABCD ][ i_DBcc ] = 1;
+	PairingArray[ i_SBCD ][ i_DBcc ] = 1;
 }
 
 
@@ -220,15 +233,20 @@ void M68000_Init(void)
 void M68000_Reset(bool bCold)
 {
 #if ENABLE_WINUAE_CPU
+	int spcFlags = regs.spcflags & (SPCFLAG_MODE_CHANGE | SPCFLAG_BRK);
 	if (bCold)
 	{
-		/* Clear registers, but we need to keep SPCFLAG_MODE_CHANGE and SPCFLAG_BRK unchanged */
-		int spcFlags = regs.spcflags & (SPCFLAG_MODE_CHANGE | SPCFLAG_BRK);
 		memset(&regs, 0, sizeof(regs));
-		regs.spcflags = spcFlags;
 	}
 	/* Now reset the WINUAE CPU core */
-	m68k_reset(bCold);
+	m68k_reset();
+
+        /* On Hatari, when we change cpu settings, we call m68k_reset() during m68k_run_xx(), */
+	/* so we must keep the value of bits SPCFLAG_MODE_CHANGE and SPCFLAG_BRK to exit m68k_run_xx() */
+	/* and choose a new m68k_run_xx() function */
+	/* [NP] TODO : don't force a reset when changing cpu settings and use common code with WinUAE ? */
+        regs.spcflags |= spcFlags;
+
 #else /* UAE CPU core */
 	if (bCold)
 	{
@@ -266,6 +284,22 @@ void M68000_Start(void)
 /*-----------------------------------------------------------------------*/
 /**
  * Check whether CPU settings have been changed.
+ * Possible values for WinUAE :
+ *	cpu_model : 68000 , 68010, 68020, 68030, 68040, 68060
+ *	cpu_level : not used anymore
+ *	cpu_compatible : 0/false (no prefetch for 68000/20/30)  1/true (prefetch opcode for 68000/20/30)
+ *	cpu_cycle_exact : 0/false   1/true (most accurate, implies cpu_compatible)
+ *	address_space_24 : 1 (68000/10 and 68030 LC for Falcon), 0 (68020/30/40/60)
+ *	fpu_model : 0, 68881 (external), 68882 (external), 68040 (cpu) , 68060 (cpu)
+ *	fpu_strict : true/false (more accurate rounding)
+ *	mmu_model : 0, 68030, 68040, 68060
+ *
+ *	m68k_speed : -1=don't adjust cycle  >=0 use m68k_speed_throttle to precisely adjust cycles
+ *	m68k_speed_throttle : if not 0, used to set cycles_mult. In Hatari, set it to 0
+ *	cpu_frequency : in CE mode, fine control of cpu freq, set it to freq/2. Not used in Hatari, set it to 0.
+ *	cpu_clock_multiplier : used to speed up/slow down clock by multiple of 2 in CE mode. In Hatari
+ *			we use nCpuFreqShift, so this should always be set to 2<<8 = 512 to get the same
+ *			cpucycleunit as in non CE mode.
  */
 void M68000_CheckCpuSettings(void)
 {
@@ -305,8 +339,22 @@ void M68000_CheckCpuSettings(void)
 	changed_prefs.cpu_cycle_exact = ConfigureParams.System.bCycleExactCpu;
 	changed_prefs.fpu_model = ConfigureParams.System.n_FPUType;
 	changed_prefs.fpu_strict = ConfigureParams.System.bCompatibleFPU;
-	changed_prefs.mmu_model = ConfigureParams.System.bMMU;
+
+	/* Update the MMU model by taking the same value as CPU model */
+	/* MMU is only supported for CPU >=68030, this is later checked in custom.c fixup_cpu() */
+	if ( !ConfigureParams.System.bMMU )
+		changed_prefs.mmu_model = 0;				/* MMU disabled */
+	else
+		changed_prefs.mmu_model = changed_prefs.cpu_model;	/* MMU enabled */
+
+	/* Set cpu speed to default values (only use in WinUAE, not in Hatari) */
+	currprefs.m68k_speed = changed_prefs.m68k_speed = 0;
+	currprefs.cpu_clock_multiplier = changed_prefs.cpu_clock_multiplier = 2 << 8;
+
+#else
+	changed_prefs.cpu_cycle_exact = 0;				/* With old UAE CPU, cycle_exact is always false */
 #endif
+
 	if (table68k)
 		check_prefs_changed_cpu();
 }
@@ -318,11 +366,43 @@ void M68000_CheckCpuSettings(void)
  */
 void M68000_MemorySnapShot_Capture(bool bSave)
 {
-	Uint32 savepc;
 #if ENABLE_WINUAE_CPU
 	int len;
-	uae_u8 *chunk = 0;
-#endif
+	uae_u8 chunk[ 1000 ];
+
+	if (bSave)
+	{
+		//m68k_dumpstate_file(stderr, NULL);
+		save_cpu (&len,chunk);
+		//printf ( "save cpu done\n"  );
+		save_cpu_extra (&len,chunk);
+		//printf ( "save cpux done\n" );
+		save_fpu (&len,chunk);
+		//printf ( "save fpu done\n"  );
+		save_mmu (&len,chunk);
+		//printf ( "save mmu done\n"  );
+		//m68k_dumpstate_file(stderr, NULL);
+	}
+	else
+	{
+		//m68k_dumpstate_file(stderr, NULL);
+		restore_cpu (chunk);
+		//printf ( "restore cpu done\n" );
+		restore_cpu_extra (chunk);
+		//printf ( "restore cpux done\n" );
+		restore_fpu (chunk);
+		//printf ( "restore fpu done\n"  );
+		restore_mmu (chunk);
+		//printf ( "restore mmu done\n"  );
+
+		restore_cpu_finish ();
+		if ( regs.s )	regs.regs[15] = regs.isp;
+		else		regs.regs[15] = regs.usp;
+		//m68k_dumpstate_file(stderr, NULL);
+	}
+
+#else /* UAE CPU core */
+	Uint32 savepc;
 
 	/* For the UAE CPU core: */
 	MemorySnapShot_Store(&currprefs.address_space_24,
@@ -338,17 +418,10 @@ void M68000_MemorySnapShot_Capture(bool bSave)
 	{
 		MemorySnapShot_Store(&savepc, sizeof(savepc));            /* PC */
 		regs.pc = savepc;
-#ifdef UAE_NEWCPU_H
 		regs.prefetch_pc = regs.pc + 128;
-#endif
 	}
 
-#ifdef UAE_NEWCPU_H
 	MemorySnapShot_Store(&regs.prefetch, sizeof(regs.prefetch));  /* prefetch */
-#else
-	uae_u32 prefetch_dummy;
-	MemorySnapShot_Store(&prefetch_dummy, sizeof(prefetch_dummy));
-#endif
 
 	if (bSave)
 	{
@@ -371,17 +444,14 @@ void M68000_MemorySnapShot_Capture(bool bSave)
 		MemorySnapShot_Store(&regs.isp, sizeof(regs.isp));
 		MemorySnapShot_Store(&regs.sr, sizeof(regs.sr));
 	}
+	MemorySnapShot_Store(&regs.opcode, sizeof(regs.opcode));
+	MemorySnapShot_Store(&regs.instruction_pc, sizeof(regs.instruction_pc));
 	MemorySnapShot_Store(&regs.stopped, sizeof(regs.stopped));
 	MemorySnapShot_Store(&regs.dfc, sizeof(regs.dfc));            /* DFC */
 	MemorySnapShot_Store(&regs.sfc, sizeof(regs.sfc));            /* SFC */
 	MemorySnapShot_Store(&regs.vbr, sizeof(regs.vbr));            /* VBR */
-#if ENABLE_WINUAE_CPU
-	MemorySnapShot_Store(&regs.caar, sizeof(regs.caar));          /* CAAR */
-	MemorySnapShot_Store(&regs.cacr, sizeof(regs.cacr));          /* CACR */
-#else
 	MemorySnapShot_Store(&caar, sizeof(caar));                    /* CAAR */
 	MemorySnapShot_Store(&cacr, sizeof(cacr));                    /* CACR */
-#endif
 	MemorySnapShot_Store(&regs.msp, sizeof(regs.msp));            /* MSP */
 
 	if (!bSave)
@@ -397,12 +467,6 @@ void M68000_MemorySnapShot_Capture(bool bSave)
 			m68k_areg(regs, 7) = regs.usp;
 	}
 
-#if ENABLE_WINUAE_CPU
-	if (bSave)
-		save_fpu(&len,0);
-	else
-		restore_fpu(chunk);
-#else
 	if (bSave)
 		save_fpu();
 	else
@@ -416,33 +480,33 @@ void M68000_MemorySnapShot_Capture(bool bSave)
  * BUSERROR - Access outside valid memory range.
  * Use bRead = 0 for write errors and bRead = 1 for read errors!
  */
-void M68000_BusError(Uint32 addr, bool bRead)
+void M68000_BusError ( Uint32 addr , int ReadWrite , int Size , int AccessType )
 {
-	/* FIXME: In prefetch mode, m68k_getpc() seems already to point to the next instruction */
-	// BusErrorPC = M68000_GetPC();		/* [NP] We set BusErrorPC in m68k_run_1 */
+	Uint32 InstrPC = M68000_InstrPC;
 
 	/* Do not print message when TOS is testing for available HW or
 	 * when a program just checks for the floating point co-processor. */
-	if ((BusErrorPC < TosAddress || BusErrorPC > TosAddress + TosSize)
+	if ((InstrPC < TosAddress || InstrPC > TosAddress + TosSize)
 	    && addr != 0xfffa42)
 	{
 		/* Print bus error message */
-		fprintf(stderr, "M68000 Bus Error %s at address $%x.\n",
-			bRead ? "reading" : "writing", addr);
+		fprintf(stderr, "M68000 Bus Error %s at address $%x PC=$%x.\n",
+			ReadWrite ? "reading" : "writing", addr, InstrPC);
 	}
 
-	if ((regs.spcflags & SPCFLAG_BUSERROR) == 0)	/* [NP] Check that the opcode has not already generated a read bus error */
+#ifndef ENABLE_WINUAE_CPU
+	if ((regs.spcflags & SPCFLAG_BUSERROR) == 0)		/* [NP] Check that the opcode has not already generated a read bus error */
 	{
 		BusErrorAddress = addr;				/* Store for exception frame */
-		bBusErrorReadWrite = bRead;
-#if ENABLE_WINUAE_CPU
-		if (currprefs.mmu_model) {
-			THROW(2);
-			return;
-		}
-#endif
+		bBusErrorReadWrite = ReadWrite;
 		M68000_SetSpecial(SPCFLAG_BUSERROR);		/* The exception will be done in newcpu.c */
 	}
+
+#else
+	/* With WinUAE's cpu, on a bus error instruction will be correctly aborted before completing, */
+	/* so we don't need to check if the opcode already generated a bus error or not */
+	exception2 ( addr , ReadWrite , Size , AccessType );
+#endif
 }
 
 
@@ -450,17 +514,16 @@ void M68000_BusError(Uint32 addr, bool bRead)
 /**
  * Exception handler
  */
-void M68000_Exception(Uint32 ExceptionVector , int ExceptionSource)
+void M68000_Exception(Uint32 ExceptionNr , int ExceptionSource)
 {
-	int exceptionNr = ExceptionVector/4;
-
+#ifndef WINUAE_FOR_HATARI
 	if ((ExceptionSource == M68000_EXC_SRC_AUTOVEC)
-		&& (exceptionNr>24 && exceptionNr<32))	/* 68k autovector interrupt? */
+		&& (ExceptionNr>24 && ExceptionNr<32))		/* 68k autovector interrupt? */
 	{
 		/* Handle autovector interrupts the UAE's way
 		 * (see intlev() and do_specialties() in UAE CPU core) */
 		/* In our case, this part is only called for HBL and VBL interrupts */
-		int intnr = exceptionNr - 24;
+		int intnr = ExceptionNr - 24;
 		pendingInterrupts |= (1 << intnr);
 		M68000_SetSpecial(SPCFLAG_INT);
 	}
@@ -477,36 +540,84 @@ void M68000_Exception(Uint32 ExceptionVector , int ExceptionSource)
 		}
 
 		/* 68k exceptions are handled by Exception() of the UAE CPU core */
-#if ENABLE_WINUAE_CPU
-		Exception(exceptionNr, m68k_getpc(), ExceptionSource);
-#else
-#ifdef UAE_NEWCPU_H
-		Exception(exceptionNr, m68k_getpc(), ExceptionSource);
-#else
-		Exception(exceptionNr, &regs, m68k_getpc(&regs));
-#endif
-#endif
-		SR = M68000_GetSR();
+		Exception(ExceptionNr, m68k_getpc(), ExceptionSource);
 
 		/* Set Status Register so interrupt can ONLY be stopped by another interrupt
-		 * of higher priority! */
-		if (ExceptionSource == M68000_EXC_SRC_INT_MFP)
+		 * of higher priority */
+		if ( (ExceptionSource == M68000_EXC_SRC_INT_MFP)
+		  || (ExceptionSource == M68000_EXC_SRC_INT_DSP) )
 		{
-			// FIXME : this test is useless, per design mfp.c will always give an address in the correct range
-			Uint32 MFPBaseVector = (unsigned int)(MFP_VR&0xf0)<<2;
-			if ( (ExceptionVector>=MFPBaseVector) && (ExceptionVector<=(MFPBaseVector+0x3c)) )
-				SR = (SR&SR_CLEAR_IPL)|0x0600; /* MFP, level 6 */
+			SR = M68000_GetSR();
+			SR = (SR&SR_CLEAR_IPL)|0x0600;		/* MFP or DSP, level 6 */
+			M68000_SetSR(SR);
 		}
-		else if (ExceptionSource == M68000_EXC_SRC_INT_DSP)
-		{
-			SR = (SR&SR_CLEAR_IPL)|0x0600;     /* DSP, level 6 */
-		}
-
-		M68000_SetSR(SR);
 	}
+
+#else
+	if ( ExceptionNr > 24 && ExceptionNr < 32 )		/* Level 1-7 interrupts */
+	{
+		/* In our case, this part is called for HBL, VBL and MFP/DSP interrupts */
+		/* (see intlev() and do_specialties() in UAE CPU core) */
+		int intnr = ExceptionNr - 24;
+		pendingInterrupts |= (1 << intnr);
+		doint();
+	}
+
+	else							/* direct CPU exceptions */
+	{
+		Exception(ExceptionNr);
+	}
+#endif
 }
 
 
+
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Update the list of pending interrupts.
+ * Level 2 (HBL) and 4 (VBL) are only cleared when the interrupt is processed,
+ * but level 6 is shared between MFP and DSP and can be cleared by MFP or DSP
+ * before being processed.
+ * So, we need to check which IRQ are set/cleared at the same time
+ * and update level 6 accordingly : level 6 = MFP_IRQ OR DSP_IRQ
+ *
+ * [NP] NOTE : temporary case for interrupts with WinUAE CPU in cycle exact mode
+ * In CE mode, interrupt state should be updated on each subcycle of every opcode
+ * then ipl_fetch() is called in each opcode.
+ * For now, Hatari with WinUAE CPU in CE mode only evaluates the interrupt state
+ * after the end of each opcode. So we need to call ipl_fetch() ourselves at the moment.
+ */
+void	M68000_Update_intlev ( void )
+{	
+#ifdef WINUAE_FOR_HATARI
+	Uint8	Level6_IRQ;
+
+#if ENABLE_DSP_EMU
+	Level6_IRQ = MFP_GetIRQ_CPU() | DSP_GetHREQ();
+#else
+	Level6_IRQ = MFP_GetIRQ_CPU();
+#endif
+	if ( Level6_IRQ == 1 )
+		pendingInterrupts |= (1 << 6);
+	else
+		pendingInterrupts &= ~(1 << 6);
+
+	if ( pendingInterrupts )
+		doint();
+	else
+		M68000_UnsetSpecial ( SPCFLAG_INT | SPCFLAG_DOINT );
+
+	/* Temporary case for WinUAE CPU in CE mode */
+	/* doint() will update regs.ipl_pin, so copy it into regs.ipl */
+	if ( ConfigureParams.System.bCycleExactCpu )
+		regs.ipl = regs.ipl_pin;			/* See ipl_fetch() in cpu/cpu_prefetch.h */
+#endif
+}
+
+
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * There seem to be wait states when a program accesses certain hardware
@@ -531,7 +642,6 @@ void M68000_WaitState(int nCycles)
  * E Clock's frequency is 1/10th of the CPU, ie 0.8 MHz in an STF/STE
  * This delay is a multiple of 2 and will follow the pattern [ 0 8 6 4 2 ]
  */
-
 int	M68000_WaitEClock ( void )
 {
 	int	CyclesToNextE;
@@ -545,3 +655,41 @@ int	M68000_WaitEClock ( void )
 
 
 
+
+/*-----------------------------------------------------------------------*/
+/**
+ * In case we modified the memory by accessing it directly (and bypassing
+ * the CPU's cache mechanism), we need to flush the instruction and data
+ * caches to force an update of the caches on the next accesses.
+ *
+ * [NP] NOTE : for now, flush_instr_caches and flush_dcache flush
+ * the whole caches, not just 'addr'
+ */
+void	M68000_Flush_All_Caches ( uaecptr addr , int size )
+{
+#ifdef WINUAE_FOR_HATARI
+	M68000_Flush_Instr_Cache ( addr , size );
+	M68000_Flush_Data_Cache ( addr , size );
+#endif
+}
+
+
+void	M68000_Flush_Instr_Cache ( uaecptr addr , int size )
+{
+#ifdef WINUAE_FOR_HATARI
+	/* Instruction cache for cpu >= 68020 */
+	flush_instr_cache ( addr , size );
+#endif
+}
+
+
+void	M68000_Flush_Data_Cache ( uaecptr addr , int size )
+{
+#ifdef WINUAE_FOR_HATARI
+	/* Data cache for cpu >= 68030 is only emulated with WinUAE */
+	flush_dcache ( addr , size );
+#endif
+}
+
+
+
