@@ -22,6 +22,7 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "joy.h"
 #include "floppy.h"
 #include "floppy_ipf.h"
+#include "floppy_stx.h"
 #include "gemdos.h"
 #include "fdc.h"
 #include "hdc.h"
@@ -410,13 +411,21 @@ static void Main_CheckForAccurateDelays(void)
 
 /* ----------------------------------------------------------------------- */
 /**
- * Set mouse pointer to new coordinates and set flag to ignore the mouse event
- * that is generated by SDL_WarpMouse().
+ * Set mouse pointer to new x,y coordinates and set flag to ignore
+ * the mouse event that is generated by SDL_WarpMouse().
+ *
+ * Skip the request is it's not position restore and mouse warping is disabled.
  */
-void Main_WarpMouse(int x, int y)
+void Main_WarpMouse(int x, int y, bool restore)
 {
-	SDL_WarpMouse(x, y);                  /* Set mouse pointer to new position */
-	bIgnoreNextMouseMotion = true;        /* Ignore mouse motion event from SDL_WarpMouse */
+	if (!(restore || ConfigureParams.Screen.bMouseWarp))
+		return;
+#if WITH_SDL2
+	SDL_WarpMouseInWindow(sdlWindow, x, y);
+#else
+	SDL_WarpMouse(x, y);
+#endif
+	bIgnoreNextMouseMotion = true;
 }
 
 
@@ -529,6 +538,7 @@ void Main_EventHandler(void)
 				/* Start double-click sequence in emulation time */
 				Keyboard.LButtonDblClk = 1;
 			}
+#if !WITH_SDL2
 			else if (event.button.button == SDL_BUTTON_WHEELDOWN)
 			{
 				/* Simulate pressing the "cursor down" key */
@@ -539,6 +549,7 @@ void Main_EventHandler(void)
 				/* Simulate pressing the "cursor up" key */
 				IKBD_PressSTKey(0x48, true);
 			}
+#endif
 			break;
 
 		 case SDL_MOUSEBUTTONUP:
@@ -550,6 +561,7 @@ void Main_EventHandler(void)
 			{
 				Keyboard.bRButtonDown &= ~BUTTON_MOUSE;
 			}
+#if !WITH_SDL2
 			else if (event.button.button == SDL_BUTTON_WHEELDOWN)
 			{
 				/* Simulate releasing the "cursor down" key */
@@ -560,6 +572,7 @@ void Main_EventHandler(void)
 				/* Simulate releasing the "cursor up" key */
 				IKBD_PressSTKey(0x48, false);
 			}
+#endif
 			break;
 
 		 case SDL_KEYDOWN:
@@ -585,21 +598,31 @@ void Main_EventHandler(void)
  */
 void Main_SetTitle(const char *title)
 {
+#if WITH_SDL2
+	if (title)
+		SDL_SetWindowTitle(sdlWindow, title);
+	else
+		SDL_SetWindowTitle(sdlWindow, PROG_NAME);
+#else
 	if (title)
 		SDL_WM_SetCaption(title, "Hatari");
 	else
 		SDL_WM_SetCaption(PROG_NAME, "Hatari");
+#endif
 }
 
 /*-----------------------------------------------------------------------*/
 /**
  * Initialise emulation for some hardware components
- * It is required to init those parts before parsing the parameters,
- * (for example, we should init FDC before inserting a disk)
+ * It is required to init those parts before parsing the parameters
+ * (for example, we should init FDC before inserting a disk and we
+ * need to know valid joysticks before selecting default joystick IDs)
  */
 static void Main_Init_HW(void)
 {
+	Joy_Init();
 	FDC_Init();
+	STX_Init();
 }
 
 /*-----------------------------------------------------------------------*/
@@ -616,8 +639,8 @@ static void Main_Init(void)
 	}
 	Log_Printf(LOG_INFO, PROG_NAME ", compiled on:  " __DATE__ ", " __TIME__ "\n");
 
-	/* Init SDL's video subsystem. Note: Audio and joystick subsystems
-	   will be initialized later (failures there are not fatal). */
+	/* Init SDL's video subsystem. Note: Audio subsystem
+	   will be initialized later (failure not fatal). */
 	if (SDL_Init(SDL_INIT_VIDEO | Opt_GetNoParachuteFlag()) < 0)
 	{
 		fprintf(stderr, "Could not initialize the SDL library:\n %s\n", SDL_GetError() );
@@ -646,7 +669,6 @@ static void Main_Init(void)
 
 	DSP_Init();
 	Floppy_Init();
-//	FDC_Init();
 	M68000_Init();                /* Init CPU emulation */
 	Audio_Init();
 	Keymap_Init();
@@ -676,7 +698,6 @@ static void Main_Init(void)
 
 	IoMem_Init();
 	NvRam_Init();
-	Joy_Init();
 	Sound_Init();
 	
 	/* done as last, needs CPU & DSP running... */
@@ -776,7 +797,7 @@ static void Main_StatusbarSetup(void)
 	Statusbar_UpdateInfo();
 }
 
-/*-----------------------------------------------------------------------*/
+
 /**
  * Main
  * 
@@ -787,6 +808,9 @@ int main(int argc, char *argv[])
 	/* Generate random seed */
 	srand(time(NULL));
 
+	/* Logs default to stderr at start */
+	Log_Default();
+
 	/* Initialize directory strings */
 	Paths_Init(argv[0]);
 
