@@ -41,6 +41,7 @@ const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " _
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "mfp.h"
+#include "midi.h"
 #include "psg.h"
 #include "reset.h"
 #include "sound.h"
@@ -53,9 +54,11 @@ const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " _
 #include "falcon/crossbar.h"
 #include "falcon/videl.h"
 #include "statusbar.h"
+#include "cart.h"
 
 
-#define VERSION_STRING      "1.8.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
+#define VERSION_STRING      "1.9.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
+#define SNAPSHOT_MAGIC      0xDeadBeef
 
 #if HAVE_LIBZ
 #define COMPRESS_MEMORYSNAPSHOT       /* Compress snapshots to reduce disk space used */
@@ -144,7 +147,7 @@ static int MemorySnapShot_fseek(MSS_File fhndl, int pos)
 #ifdef COMPRESS_MEMORYSNAPSHOT
 	return (int)gzseek(fhndl, pos, SEEK_CUR);	/* return -1 if error, new position >=0 if OK */
 #else
-	return seek(fhndl, pos, SEEK_CUR);		/* return -1 if error, 0 if OK */
+	return fseek(fhndl, pos, SEEK_CUR);		/* return -1 if error, 0 if OK */
 #endif
 }
 
@@ -157,6 +160,12 @@ static int MemorySnapShot_fseek(MSS_File fhndl, int pos)
 static bool MemorySnapShot_OpenFile(const char *pszFileName, bool bSave)
 {
 	char VersionString[] = VERSION_STRING;
+#if ENABLE_WINUAE_CPU
+# define CORE_VERSION 1
+#else
+# define CORE_VERSION 0
+#endif
+	Uint8 CpuCore;
 
 	/* Set error */
 	bCaptureError = false;
@@ -181,6 +190,9 @@ static bool MemorySnapShot_OpenFile(const char *pszFileName, bool bSave)
 		bCaptureSave = true;
 		/* Store version string */
 		MemorySnapShot_Store(VersionString, sizeof(VersionString));
+		/* Store CPU core version */
+		CpuCore = CORE_VERSION;
+		MemorySnapShot_Store(&CpuCore, sizeof(CpuCore));
 	}
 	else
 	{
@@ -197,12 +209,24 @@ static bool MemorySnapShot_OpenFile(const char *pszFileName, bool bSave)
 		/* Restore version string */
 		MemorySnapShot_Store(VersionString, sizeof(VersionString));
 		/* Does match current version? */
-		if (strcasecmp(VersionString, VERSION_STRING))
+		if (strcmp(VersionString, VERSION_STRING))
 		{
 			/* No, inform user and error */
-			Log_AlertDlg(LOG_ERROR, "Unable to restore Hatari memory state. File\n"
-			                       "is compatible only with Hatari version %s.",
-				     VersionString);
+			Log_AlertDlg(LOG_ERROR,
+				     "Unable to restore Hatari memory state.\n"
+				     "Given state file is compatible only with\n"
+				     "Hatari version " VERSION_STRING ".");
+			bCaptureError = true;
+			return false;
+		}
+		/* Check CPU core version */
+		MemorySnapShot_Store(&CpuCore, sizeof(CpuCore));
+		if (CpuCore != CORE_VERSION)
+		{
+			Log_AlertDlg(LOG_ERROR,
+				     "Unable to restore Hatari memory state.\n"
+				     "Given state file is for different Hatari\n"
+				     "CPU core version.");
 			bCaptureError = true;
 			return false;
 		}
@@ -273,6 +297,8 @@ void MemorySnapShot_Store(void *pData, int Size)
  */
 void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
 {
+	Uint32 magic = SNAPSHOT_MAGIC;
+
 	/* Set to 'saving' */
 	if (MemorySnapShot_OpenFile(pszFileName, true))
 	{
@@ -288,6 +314,7 @@ void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
 		GemDOS_MemorySnapShot_Capture(true);
 		ACIA_MemorySnapShot_Capture(true);
 		IKBD_MemorySnapShot_Capture(true);
+		MIDI_MemorySnapShot_Capture(true);
 		CycInt_MemorySnapShot_Capture(true);
 		M68000_MemorySnapShot_Capture(true);
 		MFP_MemorySnapShot_Capture(true);
@@ -301,6 +328,8 @@ void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
 		DSP_MemorySnapShot_Capture(true);
 		DebugUI_MemorySnapShot_Capture(pszFileName, true);
 		IoMem_MemorySnapShot_Capture(true);
+		/* end marker */
+		MemorySnapShot_Store(&magic, sizeof(magic));
 		/* And close */
 		MemorySnapShot_CloseFile();
 	} else {
@@ -323,6 +352,8 @@ void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
  */
 void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 {
+	Uint32 magic;
+
 	/* Set to 'restore' */
 	if (MemorySnapShot_OpenFile(pszFileName, false))
 	{
@@ -343,6 +374,7 @@ void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 		GemDOS_MemorySnapShot_Capture(false);
 		ACIA_MemorySnapShot_Capture(false);
 		IKBD_MemorySnapShot_Capture(false);			/* After ACIA */
+		MIDI_MemorySnapShot_Capture(false);
 		CycInt_MemorySnapShot_Capture(false);
 		M68000_MemorySnapShot_Capture(false);
 		MFP_MemorySnapShot_Capture(false);
@@ -357,11 +389,29 @@ void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 		DebugUI_MemorySnapShot_Capture(pszFileName, false);
 		IoMem_MemorySnapShot_Capture(false);
 
+		/* version string check catches release-to-release
+		 * state changes, bCaptureError catches too short
+		 * state file, this check a too long state file.
+		 */
+		MemorySnapShot_Store(&magic, sizeof(magic));
+		if (!bCaptureError && magic != SNAPSHOT_MAGIC)
+			bCaptureError = true;
+
 		/* And close */
 		MemorySnapShot_CloseFile();
 
+		/* Apply patches for gemdos HD if needed */
+		/* (we need to do it after cpu tables for all opcodes were rebuilt) */
+		Cart_Patch();
+
 		/* changes may affect also info shown in statusbar */
 		Statusbar_UpdateInfo();
+
+		if (bCaptureError)
+		{
+			Log_AlertDlg(LOG_ERROR, "Full memory state restore failed!\nPlease reboot emulation.");
+			return;
+		}
 	}
 
 	/* Did error? */
@@ -379,20 +429,41 @@ void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
  */
 #include <savestate.h>
 
+void save_u64(uae_u64 data)
+{
+	MemorySnapShot_Store(&data, 8);
+}
+
 void save_u32(uae_u32 data)
 {
 	MemorySnapShot_Store(&data, 4);
+//printf ("s32 %x\n", data);
 }
 
 void save_u16(uae_u16 data)
 {
 	MemorySnapShot_Store(&data, 2);
+//printf ("s16 %x\n", data);
+}
+
+void save_u8(uae_u8 data)
+{
+	MemorySnapShot_Store(&data, 1);
+//printf ("s8 %x\n", data);
+}
+
+uae_u64 restore_u64(void)
+{
+	uae_u64 data;
+	MemorySnapShot_Store(&data, 8);
+	return data;
 }
 
 uae_u32 restore_u32(void)
 {
 	uae_u32 data;
 	MemorySnapShot_Store(&data, 4);
+//printf ("r32 %x\n", data);
 	return data;
 }
 
@@ -400,5 +471,15 @@ uae_u16 restore_u16(void)
 {
 	uae_u16 data;
 	MemorySnapShot_Store(&data, 2);
+//printf ("r16 %x\n", data);
 	return data;
 }
+
+uae_u8 restore_u8(void)
+{
+	uae_u8 data;
+	MemorySnapShot_Store(&data, 1);
+//printf ("r8 %x\n", data);
+	return data;
+}
+
