@@ -30,6 +30,7 @@ const char Options_fileid[] = "Hatari options.c : " __DATE__ " " __TIME__;
 #include "floppy.h"
 #include "fdc.h"
 #include "screen.h"
+#include "statusbar.h"
 #include "sound.h"
 #include "video.h"
 #include "vdi.h"
@@ -66,6 +67,7 @@ enum {
 	OPT_GRAB,
 	OPT_FRAMESKIPS,
 	OPT_SLOWDOWN,
+	OPT_MOUSE_WARP,
 	OPT_STATUSBAR,
 	OPT_DRIVE_LED,
 	OPT_MAXWIDTH,
@@ -85,6 +87,7 @@ enum {
 	OPT_SCREEN_CROP,        /* screen capture options */
 	OPT_AVIRECORD,
 	OPT_AVIRECORD_VCODEC,
+	OPT_AVI_PNG_LEVEL,
 	OPT_AVIRECORD_FPS,
 	OPT_AVIRECORD_FILE,
 	OPT_JOYSTICK,		/* device options */
@@ -108,13 +111,18 @@ enum {
 	OPT_SLOWFLOPPY,
 	OPT_FASTFLOPPY,
 	OPT_WRITEPROT_FLOPPY,
+	OPT_HARDDRIVE,
 	OPT_WRITEPROT_HD,
 	OPT_GEMDOS_CASE,
-	OPT_HARDDRIVE,
+	OPT_GEMDOS_CONVERT,
+	OPT_GEMDOS_DRIVE,
 	OPT_ACSIHDIMAGE,
 	OPT_IDEMASTERHDIMAGE,
 	OPT_IDESLAVEHDIMAGE,
 	OPT_MEMSIZE,		/* memory options */
+#if ENABLE_WINUAE_CPU
+	OPT_TT_RAM,
+#endif
 	OPT_MEMSTATE,
 	OPT_TOS,		/* ROM options */
 	OPT_PATCHTOS,
@@ -203,6 +211,8 @@ static const opt_t HatariOptions[] = {
 	  "<x>", "Skip <x> frames after each shown frame (0=off, >4=auto/max)" },
 	{ OPT_SLOWDOWN, NULL, "--slowdown",
 	  "<x>", "VBL wait time multiplier (1-8, default 1)" },
+	{ OPT_MOUSE_WARP, NULL, "--mousewarp",
+	  "<bool>", "Center host mouse on reset & resolution changes" },
 	{ OPT_STATUSBAR, NULL, "--statusbar",
 	  "<bool>", "Show statusbar (floppy leds etc)" },
 	{ OPT_DRIVE_LED,   NULL, "--drive-led",
@@ -248,11 +258,13 @@ static const opt_t HatariOptions[] = {
 	{ OPT_AVIRECORD, NULL, "--avirecord",
 	  NULL, "Start AVI recording" },
 	{ OPT_AVIRECORD_VCODEC, NULL, "--avi-vcodec",
-	  "<x>", "Select avi video codec (x = bmp/png)" },
+	  "<x>", "Select AVI video codec (x = bmp/png)" },
+	{ OPT_AVI_PNG_LEVEL, NULL, "--png-level",
+	  "<x>", "Select AVI PNG compression level (x = 0-9)" },
 	{ OPT_AVIRECORD_FPS, NULL, "--avi-fps",
-	  "<x>", "Force avi frame rate (x = 50/60/71/...)" },
+	  "<x>", "Force AVI frame rate (x = 50/60/71/...)" },
 	{ OPT_AVIRECORD_FILE, NULL, "--avi-file",
-	  "<file>", "Use <file> to record avi" },
+	  "<file>", "Use <file> to record AVI" },
 
 	{ OPT_HEADER, NULL, NULL, NULL, "Devices" },
 	{ OPT_JOYSTICK,  "-j", "--joystick",
@@ -285,7 +297,7 @@ static const opt_t HatariOptions[] = {
 	{ OPT_RS232_OUT, NULL, "--rs232-out",
 	  "<file>", "Enable serial port and use <file> as the output device" },
 	
-	{ OPT_HEADER, NULL, NULL, NULL, "Disk" },
+	{ OPT_HEADER, NULL, NULL, NULL, "Floppy drive" },
 	{ OPT_DRIVEA, NULL, "--drive-a",
 	  "<bool>", "Enable/disable drive A (default is on)" },
 	{ OPT_DRIVEB, NULL, "--drive-b",
@@ -304,14 +316,20 @@ static const opt_t HatariOptions[] = {
 	  "<bool>", "Speed up floppy disk access emulation (can break some programs)" },
 	{ OPT_WRITEPROT_FLOPPY, NULL, "--protect-floppy",
 	  "<x>", "Write protect floppy image contents (on/off/auto)" },
+
+	{ OPT_HEADER, NULL, NULL, NULL, "Hard drive" },
+	{ OPT_HARDDRIVE, "-d", "--harddrive",
+	  "<dir>", "Emulate harddrive partition(s) with <dir> contents" },
 	{ OPT_WRITEPROT_HD, NULL, "--protect-hd",
 	  "<x>", "Write protect harddrive <dir> contents (on/off/auto)" },
 	{ OPT_GEMDOS_CASE, NULL, "--gemdos-case",
 	  "<x>", "Forcibly up/lowercase new GEMDOS dir/filenames (off/upper/lower)" },
-	{ OPT_HARDDRIVE, "-d", "--harddrive",
-	  "<dir>", "Emulate harddrive partition(s) with <dir> contents" },
+	{ OPT_GEMDOS_CONVERT, NULL, "--gemdos-conv",
+	  "<bool>", "Atari GEMDOS <-> host (UTF-8) file name conversion" },
+	{ OPT_GEMDOS_DRIVE, NULL, "--gemdos-drive",
+	  "<drive>", "Assign GEMDOS HD <dir> to drive letter <drive> (C-Z, skip)" },
 	{ OPT_ACSIHDIMAGE,   NULL, "--acsi",
-	  "<file>", "Emulate an ACSI harddrive with an image <file>" },
+	  "<id>=<file>", "Emulate an ACSI harddrive (0-7) with an image <file>" },
 	{ OPT_IDEMASTERHDIMAGE,   NULL, "--ide-master",
 	  "<file>", "Emulate an IDE master harddrive with an image <file>" },
 	{ OPT_IDESLAVEHDIMAGE,   NULL, "--ide-slave",
@@ -320,6 +338,10 @@ static const opt_t HatariOptions[] = {
 	{ OPT_HEADER, NULL, NULL, NULL, "Memory" },
 	{ OPT_MEMSIZE,   "-s", "--memsize",
 	  "<x>", "ST RAM size (x = size in MiB from 0 to 14, 0 = 512KiB)" },
+#if ENABLE_WINUAE_CPU
+	{ OPT_TT_RAM,   NULL, "--ttram",
+	  "<x>", "TT RAM size (x = size in MiB from 0 to 256)" },
+#endif
 	{ OPT_MEMSTATE,   NULL, "--memstate",
 	  "<file>", "Load memory snap-shot <file>" },
 
@@ -604,6 +626,28 @@ static bool Opt_ShowError(unsigned int optid, const char *value, const char *err
 }
 
 
+/**
+ * Return given value after constraining it within "min" and "max" values
+ * and making it evenly divisable by "align"
+ */
+int Opt_ValueAlignMinMax(int value, int align, int min, int max)
+{
+	value = (value/align)*align;
+	if (value > max)
+	{
+		/* align down */
+		return (max/align)*align;
+	}
+	if (value < min)
+	{
+		/* align up */
+		min += align-1;
+		return (min/align)*align;
+	}
+	return value;
+}
+
+
 /**
  * If 'conf' given, set it:
  * - true if given option 'arg' is y/yes/on/true/1
@@ -682,6 +726,10 @@ static int Opt_CheckBracketValue(const opt_t *opt, const char *str)
 	{
 		return OPT_CONTINUE;
 	}
+	if (str[offset+1])
+	{
+		return OPT_CONTINUE;
+	}
 	optstr = opt->str;
 	for (i = 0; opt->str == optstr; opt++, i++)
 	{
@@ -777,7 +825,7 @@ static bool Opt_StrCpy(int optid, bool checkexist, char *dst, const char *src, s
 	}
 	if (checkexist && !File_Exists(src))
 	{
-		return Opt_ShowError(optid, src, "Given file doesn't exist (or has wrong file permissions)!");
+		return Opt_ShowError(optid, src, "Given file doesn't exist or permissions prevent access to it!");
 	}
 	if (option)
 	{
@@ -803,6 +851,26 @@ Uint32 Opt_GetNoParachuteFlag(void)
 }
 
 
+/**
+ * Return true if given path points to an Atari program, false otherwise.
+ */
+bool Opt_IsAtariProgram(const char *path)
+{
+	bool ret = false;
+	Uint8 test[2];
+	FILE *fp;
+
+	if (File_Exists(path) && (fp = fopen(path, "rb"))) {
+		/* file starts with GEMDOS magic? */
+		if (fread(test, 1, 2, fp) == 2 &&
+		    test[0] == 0x60 && test[1] == 0x1A) {
+			ret = true;
+		}
+		fclose(fp);
+	}
+	return ret;
+}
+
 /**
  * Handle last (non-option) argument.  It can be a path or filename.
  * Filename can be a disk image or Atari program.
@@ -811,32 +879,23 @@ Uint32 Opt_GetNoParachuteFlag(void)
 static bool Opt_HandleArgument(const char *path)
 {
 	char *dir = NULL;
-	Uint8 test[2];
-	FILE *fp;
 
 	/* Atari program? */
-	if (File_Exists(path) && (fp = fopen(path, "rb"))) {
-
-		/* file starts with GEMDOS magic? */
-		if (fread(test, 1, 2, fp) == 2 &&
-		    test[0] == 0x60 && test[1] == 0x1A) {
-
-			const char *prgname = strrchr(path, PATHSEP);
-			if (prgname) {
-				dir = strdup(path);
-				dir[prgname-path] = '\0';
-				prgname++;
-			} else {
-				dir = strdup(Paths_GetWorkingDir());
-				prgname = path;
-			}
-			/* after above, dir should point to valid dir,
-			 * then make sure that given program from that
-			 * dir will be started.
-			 */
-			TOS_AutoStart(prgname);
+	if (Opt_IsAtariProgram(path)) {
+		const char *prgname = strrchr(path, PATHSEP);
+		if (prgname) {
+			dir = strdup(path);
+			dir[prgname-path] = '\0';
+			prgname++;
+		} else {
+			dir = strdup(Paths_GetWorkingDir());
+			prgname = path;
 		}
-		fclose(fp);
+		/* after above, dir should point to valid dir,
+		 * then make sure that given program from that
+		 * dir will be started.
+		 */
+		TOS_AutoStart(prgname);
 	}
 	if (dir) {
 		path = dir;
@@ -882,8 +941,8 @@ static bool Opt_HandleArgument(const char *path)
  */
 bool Opt_ParseParameters(int argc, const char * const argv[])
 {
-	int ncpu, skips, zoom, planes, cpuclock, threshold, memsize, port, freq, temp;
-	const char *errstr;
+	int ncpu, skips, zoom, planes, cpuclock, threshold, memsize, port, freq, temp, drive;
+	const char *errstr, *str;
 	int i, ok = true;
 	int val;
 
@@ -996,6 +1055,10 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			break;
 
+		case OPT_MOUSE_WARP:
+			ok = Opt_Bool(argv[++i], OPT_MOUSE_WARP, &ConfigureParams.Screen.bMouseWarp);
+			break;
+
 		case OPT_STATUSBAR:
 			ok = Opt_Bool(argv[++i], OPT_STATUSBAR, &ConfigureParams.Screen.bShowStatusbar);
 			break;
@@ -1049,16 +1112,14 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			{
 				return Opt_ShowError(OPT_ZOOM, argv[i], "Invalid zoom value");
 			}
+			ConfigureParams.Screen.nMaxWidth = NUM_VISIBLE_LINE_PIXELS;
+			ConfigureParams.Screen.nMaxHeight = NUM_VISIBLE_LINES;
 			if (zoom > 1)
 			{
-				ConfigureParams.Screen.nMaxWidth = 2*(48+320+48);
-				ConfigureParams.Screen.nMaxHeight = 2*NUM_VISIBLE_LINES+24;
-			}
-			else
-			{
-				ConfigureParams.Screen.nMaxWidth = 1*(48+320+48);
-				ConfigureParams.Screen.nMaxHeight = 1*NUM_VISIBLE_LINES+12;
+				ConfigureParams.Screen.nMaxWidth *= 2;
+				ConfigureParams.Screen.nMaxHeight *= 2;
 			}
+			ConfigureParams.Screen.nMaxHeight += STATUSBAR_MAX_HEIGHT;
 			break;
 
 			/* Falcon/TT display options */
@@ -1107,6 +1168,12 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			}
 			break;
 
+		case OPT_AVI_PNG_LEVEL:
+			i += 1;
+			if (!Avi_SetCompressionLevel(argv[i]))
+				return Opt_ShowError(OPT_AVI_PNG_LEVEL, argv[i], "Invalid compression level");
+			break;
+
 		case OPT_AVIRECORD_FPS:
 			val = atoi(argv[++i]);
 			if (val < 0 || val > 100)
@@ -1183,7 +1250,10 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_JOYSTICK4:
 		case OPT_JOYSTICK5:
 			port = argv[i][strlen(argv[i])-1] - '0';
-			assert(port >= 0 && port < JOYSTICK_COUNT);
+			if (port < 0 || port >= JOYSTICK_COUNT)
+			{
+				return Opt_ShowError(OPT_JOYSTICK0, argv[i], "Invalid joystick port");
+			}
 			i += 1;
 			if (strcasecmp(argv[i], "none") == 0)
 			{
@@ -1330,6 +1400,29 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 				return Opt_ShowError(OPT_GEMDOS_CASE, argv[i], "Unknown option value");
 			break;
 
+		case OPT_GEMDOS_CONVERT:
+			ok = Opt_Bool(argv[++i], OPT_GEMDOS_CONVERT, &ConfigureParams.HardDisk.bFilenameConversion);
+			break;
+
+		case OPT_GEMDOS_DRIVE:
+			i += 1;
+			if (strcasecmp(argv[i], "skip") == 0)
+			{
+				ConfigureParams.HardDisk.nGemdosDrive = DRIVE_SKIP;
+				break;
+			}
+			else if (strlen(argv[i]) == 1)
+			{
+				int drive = toupper(argv[i][0]);
+				if (drive >= 'C' && drive <= 'Z')
+				{
+					drive = drive - 'C' + DRIVE_C;
+					ConfigureParams.HardDisk.nGemdosDrive = drive;
+					break;
+				}
+			}
+			return Opt_ShowError(OPT_GEMDOS_DRIVE, argv[i], "Invalid <drive>");
+
 		case OPT_HARDDRIVE:
 			i += 1;
 			ok = Opt_StrCpy(OPT_HARDDRIVE, false, ConfigureParams.HardDisk.szHardDiskDirectories[0],
@@ -1350,9 +1443,21 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 
 		case OPT_ACSIHDIMAGE:
 			i += 1;
-			ok = Opt_StrCpy(OPT_ACSIHDIMAGE, true, ConfigureParams.Acsi[0].sDeviceFile,
-					argv[i], sizeof(ConfigureParams.Acsi[0].sDeviceFile),
-					&ConfigureParams.Acsi[0].bUseDevice);
+			str = argv[i];
+			if (strlen(str) > 2 && isdigit(str[0]) && str[1] == '=')
+			{
+				drive = str[0] - '0';
+				if (drive < 0 || drive > 7)
+					return Opt_ShowError(OPT_ACSIHDIMAGE, str, "Invalid ACSI drive <id>, must be 0-7");
+				str += 2;
+			}
+			else
+			{
+				drive = 0;
+			}
+			ok = Opt_StrCpy(OPT_ACSIHDIMAGE, true, ConfigureParams.Acsi[drive].sDeviceFile,
+					str, sizeof(ConfigureParams.Acsi[drive].sDeviceFile),
+					&ConfigureParams.Acsi[drive].bUseDevice);
 			if (ok)
 			{
 				bLoadAutoSave = false;
@@ -1392,6 +1497,14 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 			bLoadAutoSave = false;
 			break;
 
+#if ENABLE_WINUAE_CPU
+		case OPT_TT_RAM:
+			memsize = atoi(argv[++i]);
+			ConfigureParams.Memory.nTTRamSize = Opt_ValueAlignMinMax(memsize+3, 4, 0, 256);
+			bLoadAutoSave = false;
+			break;
+#endif
+
 		case OPT_TOS:
 			i += 1;
 			ok = Opt_StrCpy(OPT_TOS, true, ConfigureParams.Rom.szTosImageFileName,
@@ -1434,10 +1547,16 @@ bool Opt_ParseParameters(int argc, const char * const argv[])
 		case OPT_CPULEVEL:
 			/* UAE core uses cpu_level variable */
 			ncpu = atoi(argv[++i]);
+#if ENABLE_WINUAE_CPU
+			if(ncpu < 0 || ncpu == 5 || ncpu > 6)
+#else
 			if(ncpu < 0 || ncpu > 4)
+#endif
 			{
 				return Opt_ShowError(OPT_CPULEVEL, argv[i], "Invalid CPU level");
 			}
+			if ( ncpu == 6 )			/* Special case for 68060, nCpuLevel should be 5 */
+				ncpu = 5;
 			ConfigureParams.System.nCpuLevel = ncpu;
 			bLoadAutoSave = false;
 			break;
