@@ -130,9 +130,9 @@ const char PSG_fileid[] = "Hatari psg.c : " __DATE__ " " __TIME__;
 #include "fdc.h"
 
 
-Uint8 PSGRegisterSelect;        /* Write to 0xff8800 sets the register number used in read/write accesses */
-Uint8 PSGRegisterReadData;	/* Value returned when reading from 0xff8800 */
-Uint8 PSGRegisters[MAX_PSG_REGISTERS]; /* Registers in PSG, see PSG_REG_xxxx */
+static Uint8 PSGRegisterSelect;		/* Write to 0xff8800 sets the register number used in read/write accesses */
+static Uint8 PSGRegisterReadData;	/* Value returned when reading from 0xff8800 */
+Uint8 PSGRegisters[MAX_PSG_REGISTERS];	/* Registers in PSG, see PSG_REG_xxxx */
 
 static unsigned int LastStrobe=0; /* Falling edge of Strobe used for printer */
 
@@ -351,15 +351,17 @@ void PSG_Set_DataRegister(Uint8 val)
 		/* Report a possible drive/side change */
 		FDC_SetDriveSide ( val_old & 7 , PSGRegisters[PSG_REG_IO_PORTA] & 7 );
 
-		/* Bit 3 - Centronics as input */
-		if(PSGRegisters[PSG_REG_IO_PORTA]&(1<<3))
-		{
-			/* FIXME: might be needed if we want to emulate sound sampling hardware */
-		}
-		
 		/* handle Falcon specific bits in PORTA of the PSG */
 		if (ConfigureParams.System.nMachineType == MACHINE_FALCON)
 		{
+			/* Bit 3 - centronics port SELIN line (pin 17) */
+			/*
+			if (PSGRegisters[PSG_REG_IO_PORTA] & (1 << 3))
+			{
+				// not emulated yet
+			}
+			*/
+
 			/* Bit 4 - DSP reset? */
 			if(PSGRegisters[PSG_REG_IO_PORTA]&(1<<4))
 			{
@@ -556,11 +558,11 @@ void PSG_ff8803_WriteByte(void)
 /* ------------------------------------------------------------------
  * YM-2149 register content dump (for debugger info command)
  */
-void PSG_Info(Uint32 dummy)
+void PSG_Info(FILE *fp, Uint32 dummy)
 {
 	int i;
 	for(i = 0; i < ARRAYSIZE(PSGRegisters); i++)
 	{
-		fprintf(stderr, "Reg $%02X : $%02X\n", i, PSGRegisters[i]);
+		fprintf(fp, "Reg $%02X : $%02X\n", i, PSGRegisters[i]);
 	}
 }
