@@ -48,6 +48,7 @@ const char Statusbar_fileid[] = "Hatari statusbar.c : " __DATE__ " " __TIME__;
 #include "avi_record.h"
 #include "vdi.h"
 #include "fdc.h"
+#include "stMemory.h"
 
 #define DEBUG 0
 #if DEBUG
@@ -99,7 +100,7 @@ static Uint32 RecColorOn, RecColorOff;
 static Uint32 GrayBg, LedColorBg;
 
 /* needs to be enough for all messages, but <= MessageRect width / font width */
-#define MAX_MESSAGE_LEN 52
+#define MAX_MESSAGE_LEN 60
 typedef struct msg_item {
 	struct msg_item *next;
 	char msg[MAX_MESSAGE_LEN+1];
@@ -131,7 +132,9 @@ static int StatusbarHeight;
 int Statusbar_GetHeightForSize(int width, int height)
 {
 	int h = 0;
-	/* Must arrive at same conclusion about font size as SDLGui_SetScreen() */
+	/* Must arrive at same conclusion about font size as SDLGui_SetScreen(),
+	 * and max size returned by this must correspond to STATUSBAR_MAX_HEIGHT
+	 */
 	if (ConfigureParams.Screen.bShowStatusbar) {
 		/* smaller SDL GUI font height = 8, larger = 16 */
 		h = 8;
@@ -426,18 +429,22 @@ void Statusbar_UpdateInfo(void)
 		*end++ = '0' + ConfigureParams.System.nCpuFreq / 10;
 	}
 	*end++ = '0' + ConfigureParams.System.nCpuFreq % 10;
-	end = Statusbar_AddString(end, "MHz/");
+	end = Statusbar_AddString(end, "MHz");
 
 	/* CPU type */
 	if(ConfigureParams.System.nCpuLevel > 0) {
-		*end++ = '0';
-		*end++ = '0' + ConfigureParams.System.nCpuLevel % 10;
-		*end++ = '0';
 		*end++ = '/';
+		*end++ = '0';
+		if ( ConfigureParams.System.nCpuLevel == 5 )	/* Special case : 68060 has nCpuLevel=5 */
+			*end++ = '0' + 6;
+		else
+			*end++ = '0' + ConfigureParams.System.nCpuLevel % 10;
+		*end++ = '0';
 	}
 
 	/* additional WinUAE CPU/FPU info */
 #if ENABLE_WINUAE_CPU
+	*end++ = '/';
 	switch (ConfigureParams.System.n_FPUType) {
 	case FPU_68881:
 		end = Statusbar_AddString(end, "68881");
@@ -451,13 +458,13 @@ void Statusbar_UpdateInfo(void)
 	default:
 		*end++ = '-';
 	}
-	*end++ = '/';
 	if (ConfigureParams.System.bMMU) {
-		end = Statusbar_AddString(end, "MMU/");
+		end = Statusbar_AddString(end, "/MMU");
 	}
 #endif
 
 	/* amount of memory */
+	*end++ = ' ';
 	if (ConfigureParams.Memory.nMemorySize > 9) {
 		*end++ = '1';
 		*end++ = '0' + ConfigureParams.Memory.nMemorySize % 10;
@@ -468,6 +475,9 @@ void Statusbar_UpdateInfo(void)
 			end = Statusbar_AddString(end, "0.5");
 		}
 	}
+	if (TTmemory && ConfigureParams.Memory.nTTRamSize) {
+		end += sprintf(end, "/%i", ConfigureParams.Memory.nTTRamSize);
+	}
 	end = Statusbar_AddString(end, "MB ");
 
 	/* machine type */
@@ -589,12 +599,8 @@ static SDL_Rect* Statusbar_ShowMessage(SDL_Surface *surf, Uint32 ticks)
 		assert(MessageList->next); /* last message shouldn't end here */
 		next = MessageList->next;
 		free(MessageList);
+		/* show next */
 		MessageList = next;
-		/* make sure next message gets shown */
-		MessageList->shown = false;
-	}
-	if (MessageList->shown) {
-		return NULL;
 	}
 	/* not shown yet, show */
 	MessageList->shown = true;
