@@ -83,8 +83,8 @@ static bool XBios_Floprd(Uint32 Params)
 	Count = STMemory_ReadWord(Params+SIZE_LONG+SIZE_LONG+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD);
 
 	LOG_TRACE(TRACE_OS_XBIOS, "XBIOS 0x08 Floprd(0x%x, %d, %d, %d, %d, %d) at PC 0x%X for: %s\n",
-		  pBuffer, Dev, Sector, Track, Side, Count,
-		  M68000_GetPC(), EmulationDrives[Dev].sFileName);
+		  pBuffer, Dev, Sector, Track, Side, Count, M68000_GetPC(),
+		  Dev < MAX_FLOPPYDRIVES ? EmulationDrives[Dev].sFileName : "n/a");
 	return false;
 }
 
@@ -107,8 +107,8 @@ static bool XBios_Flopwr(Uint32 Params)
 	Count = STMemory_ReadWord(Params+SIZE_LONG+SIZE_LONG+SIZE_WORD+SIZE_WORD+SIZE_WORD+SIZE_WORD);
 
 	LOG_TRACE(TRACE_OS_XBIOS, "XBIOS 0x09 Flopwr(0x%x, %d, %d, %d, %d, %d) at PC 0x%X for: %s\n",
-		  pBuffer, Dev, Sector, Track, Side, Count,
-		  M68000_GetPC(), EmulationDrives[Dev].sFileName);
+		  pBuffer, Dev, Sector, Track, Side, Count, M68000_GetPC(),
+		  Dev < MAX_FLOPPYDRIVES ? EmulationDrives[Dev].sFileName : "n/a");
 	return false;
 }
 
@@ -219,7 +219,7 @@ static bool XBios_Scrdmp(Uint32 Params)
 static bool XBios_HatariControl(Uint32 Params)
 {
 	const char *pText;
-	pText = (const char *)STRAM_ADDR(STMemory_ReadLong(Params));
+	pText = (const char *)STMemory_STAddrToPointer(STMemory_ReadLong(Params));
 	LOG_TRACE(TRACE_OS_XBIOS, "XBIOS 0x%02X HatariControl(%s) at PC 0x%X\n", HATARI_CONTROL_OPCODE, pText, M68000_GetPC());
 
 	if (!bXBiosCommands)
@@ -414,21 +414,21 @@ static const char* XBios_Call2Name(Uint16 opcode)
 	return "???";
 }
 
-void XBios_Info(Uint32 dummy)
+void XBios_Info(FILE *fp, Uint32 dummy)
 {
 	Uint16 opcode;
 	for (opcode = 0; opcode < 168; ) {
-		fprintf(stderr, "%02x %-21s", opcode,
+		fprintf(fp, "%02x %-21s", opcode,
 			XBios_Call2Name(opcode));
 		if (++opcode % 3 == 0) {
-			fputs("\n", stderr);
+			fputs("\n", fp);
 		}
 	}
 }
 #else /* !ENABLE_TRACING */
-void XBios_Info(Uint32 bShowOpcodes)
+void XBios_Info(FILE *fp, Uint32 bShowOpcodes)
 {
-	        fputs("Hatari isn't configured with ENABLE_TRACING\n", stderr);
+	        fputs("Hatari isn't configured with ENABLE_TRACING\n", fp);
 }
 #endif /* !ENABLE_TRACING */
 
